# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from datetime import datetime
from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateTransition, StateView, Button
from trytond.transaction import Transaction
from trytond.pool import Pool
from trytond.pyson import Eval
from trytond.i18n import gettext
from ..exceptions import LabOrderExists

__all__ = [
    'CreateLabTestOrderInit', 'CreateLabTestOrder', 'RequestTest',
    'RequestPatientLabTestStart', 'RequestPatientLabTest']


from trytond.modules.health.core import (get_health_professional,
                                         get_age_for_comparison)


class CreateLabTestOrderInit(ModelView):
    'Create Test Report Init'
    __name__ = 'gnuhealth.lab.test.create.init'


class CreateLabTestOrder(Wizard):
    'Create Lab Test Report'
    __name__ = 'gnuhealth.lab.test.create'

    start = StateView(
        'gnuhealth.lab.test.create.init',
        'health_lab.view_lab_make_test', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Create Test Order', 'create_lab_test', 'tryton-ok', True),
        ])

    create_lab_test = StateTransition()

    def transition_create_lab_test(self):
        TestRequest = Pool().get('gnuhealth.patient.lab.test')
        Lab = Pool().get('gnuhealth.lab')

        tests_report_data = []

        tests = TestRequest.browse(Transaction().context.get('active_ids'))

        for lab_test_order in tests:

            test_cases = []
            test_report_data = {}

            if lab_test_order.state == 'ordered':
                raise LabOrderExists(
                    gettext('health_lab.msg_lab_order_exists')
                )

            test_report_data['test'] = lab_test_order.test_type.id
            test_report_data['source_type'] = lab_test_order.source_type
            test_report_data['specimen_type'] = (lab_test_order
                                                 .test_type.specimen_type)
            test_report_data['patient'] = (lab_test_order.patient_id
                                           and lab_test_order.patient_id.id)
            test_report_data['other_source'] = lab_test_order.other_source
            if lab_test_order.doctor_id:
                test_report_data['requestor'] = lab_test_order.doctor_id.id
            test_report_data['date_requested'] = lab_test_order.date
            test_report_data['request_order'] = lab_test_order.request

            for critearea in lab_test_order.test_type.critearea:
                test_cases.append(('create', [{
                    'name': critearea.name,
                    'code': critearea.code,
                    'test_method': critearea.test_method,
                    'sequence': critearea.sequence,
                    'limits_verified': critearea.limits_verified,
                    'lower_limit': critearea.lower_limit,
                    'upper_limit': critearea.upper_limit,
                    'normal_range': critearea.normal_range,
                    "to_integer": critearea.to_integer,
                    'units': critearea.units and critearea.units.id,
                }]))
            test_report_data['critearea'] = test_cases

            tests_report_data.append(test_report_data)

        Lab.create(tests_report_data)
        TestRequest.write(tests, {'state': 'ordered'})

        return 'end'


class RequestTest(ModelView):
    'Request - Test'
    __name__ = 'gnuhealth.request-test'
    _table = 'gnuhealth_request_test'

    request = fields.Many2One(
        'gnuhealth.patient.lab.test.request.start',
        'Request', required=True)
    test = fields.Many2One('gnuhealth.lab.test_type', 'Test', required=True)


class RequestPatientLabTestStart(ModelView):
    'Request Patient Lab Test Start'
    __name__ = 'gnuhealth.patient.lab.test.request.start'

    date = fields.DateTime('Date')
    source_type = fields.Selection([
        ('patient', 'Patient'),
        ('other_source', 'Other')
    ], 'Source',
        help='Sample source type.',
        sort=False)
    patient = fields.Many2One(
        'gnuhealth.patient', 'Patient',
        states={'invisible': (Eval('source_type') != 'patient')})
    other_source = fields.Char(
        'Other',
        states={
            'invisible': (
                Eval('source_type') != 'other_source')},
        help="Other sample source.")

    gender_str = fields.Char(
        'Gender', readonly=True,
        states={'invisible': (Eval('source_type') != 'patient')})

    age_num = fields.Float(
        'Age Num',
        digits=(3, 3), readonly=True,
        help='Age year number, '
        '(years x 365 + months x 30.5 + days) / 365',
        states={'invisible': (Eval('source_type') != 'patient')})

    context = fields.Many2One(
        'gnuhealth.pathology', 'Context',
        help="Health context for this order. It can be a suspected or"
             " existing health condition, a regular health checkup, ...")
    doctor = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof',
        help="Health professional who ordered the lab tests.")
    tests = fields.Many2Many(
        'gnuhealth.request-test', 'request', 'test',
        'Tests', required=True)
    urgent = fields.Boolean('Urgent')

    @staticmethod
    def default_date():
        return datetime.now()

    @staticmethod
    def default_source_type():
        return 'patient'

    @staticmethod
    def default_patient():
        if Transaction().context.get('active_model') == 'gnuhealth.patient':
            return Transaction().context.get('active_id')

    @staticmethod
    def default_doctor():
        return get_health_professional()

    # Update age_num and gender_str based on the patient
    @fields.depends('patient')
    def on_change_patient(self):
        if (self.patient):
            self.gender_str = self.patient.gender_str
            self.age_num = get_age_for_comparison(
                self.patient.age, type='y')
        else:
            self.gender_str = None
            self.age_num = None

    @classmethod
    def __setup__(cls):
        super(RequestPatientLabTestStart, cls).__setup__()

        # Do not cache default_key as it depends on time
        cls.__rpc__['default_get'].cache = None


class RequestPatientLabTest(Wizard):
    'Request Patient Lab Test'
    __name__ = 'gnuhealth.patient.lab.test.request'

    start = StateView(
        'gnuhealth.patient.lab.test.request.start',
        'health_lab.patient_lab_test_request_start_view_form', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Request', 'request', 'tryton-ok', default=True),
        ])
    request = StateTransition()

    def generate_code(self, **pattern):
        Config = Pool().get('gnuhealth.sequences')
        config = Config(1)
        sequence = config.get_multivalue(
            'lab_request_sequence', **pattern)
        if sequence:
            return sequence.get()

    def transition_request(self):
        PatientLabTest = Pool().get('gnuhealth.patient.lab.test')
        request_number = self.generate_code()
        lab_tests = []
        for test in self.start.tests:
            lab_test = {}
            lab_test['request'] = request_number
            lab_test['test_type'] = test.id
            lab_test['source_type'] = self.start.source_type
            lab_test['patient_id'] = (self.start.patient
                                      and self.start.patient.id)
            lab_test['other_source'] = self.start.other_source
            lab_test['specimen_type'] = test.specimen_type
            if self.start.doctor:
                lab_test['doctor_id'] = self.start.doctor.id
            if self.start.context:
                lab_test['context'] = self.start.context.id
            lab_test['date'] = self.start.date
            lab_test['urgent'] = self.start.urgent
            lab_tests.append(lab_test)
        PatientLabTest.create(lab_tests)

        return 'end'



# Copyright (C) 2008-2025 Luis Falcon <falcon@gnuhealth.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import pytz
from datetime import datetime
from trytond.pool import Pool
from trytond.transaction import Transaction
from trytond.report import Report

__all__ = ['SurgeryReport']


class SurgeryReport(Report):
    __name__ = 'surgery'

    @classmethod
    def parse(cls, report, objects, data, localcontext):
        Company = Pool().get('company.company')

        timezone = None
        company_id = Transaction().context.get('company')
        if company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)

        dt = datetime.now()

        localcontext['timezone'] = timezone
        localcontext['datetime_now'] = dt

        return super(SurgeryReport, cls).parse(
            report, objects, data, localcontext)



# SPDX-FileCopyrightText:  2024 - Wei Zhao <wei.zhao@uclouvain.be>
# SPDX-License-Identifier: GPL-3.0-or-later


from trytond.model import ModelView
from trytond.wizard import Wizard, StateView, StateTransition, Button
from trytond.pool import Pool
import logging

__all__ = ['FullSynchronizeStart', 'FullSynchronize']

logger = logging.getLogger(__name__)


class FullSynchronizeStart(ModelView):
    """
    Full Synchronize studies Start
    """
    __name__ = "gnuhealth.imaging_orthanc.full_synchronize.start"

#
# This class is responsible for retrieving all
# saved studies from the Orthanc server.


class FullSynchronize(Wizard):
    "Full Synchronize Studies"
    __name__ = 'gnuhealth.imaging_orthanc.full_synchronize'

    start = StateView(
        'gnuhealth.imaging_orthanc.full_synchronize.start',
        'health_orthanc.full_synchronize_start_form', [
            Button("Cancel", 'end', 'tryton-cancel'),
            Button("Synchronize", 'synchronize', 'tryton-ok'),
        ])
    synchronize = StateTransition()

    def transition_synchronize(self):
        """
        Full Synchronize studies and return 'end'.
        """
        Pool().get('gnuhealth.imaging_orthanc.study').full_synchronize()
        return 'end'

    def end(self):
        """
        Method to signal the end of the process and return the string 'reload'.
        """
        return 'reload'



# Copyright (C) 2008-2025 Luis Falcon <lfalcon@gnusolidario.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import datetime
import decimal
from trytond.wizard import Wizard
from trytond.transaction import Transaction
from trytond.pool import Pool
from trytond.i18n import gettext
from trytond.modules.product import round_price
from ..exceptions import (
    ServiceInvoiced, NoInvoiceAddress, NoPaymentTerm, NoAccountReceivable)

__all__ = ['CreateServiceInvoice']


class CreateServiceInvoice(Wizard):
    __name__ = 'gnuhealth.service.invoice.create'

#  name: CreateServiceInvoice.discount_policy
#  @param : Insurance, service line product
#  @return : Policy applied to that service line

    def discount_policy(self, insurance, product):
        # Check that there is a plan associated to the insurance
        if insurance.plan_id:
            # Traverse the product policies within the plan
            # In terms of applying discount, category and product are
            # mutually exclusive.

            discount = {}
            # If the policy line contains both values (percentage and
            # fixed price, the percentage value will take over.
            if insurance.plan_id.product_policy:
                for policy in insurance.plan_id.product_policy:
                    # Check first for product
                    if (product == policy.product):
                        if policy.discount:
                            discount['value'] = policy.discount
                            discount['type'] = 'pct'
                            return discount

                        if policy.price:
                            discount['value'] = policy.price
                            discount['type'] = 'fixed'
                            return discount

                for policy in insurance.plan_id.product_policy:
                    # Then, if there's no product, check for the category
                    if (policy.product_category in product.categories):
                        if policy.discount:
                            discount['value'] = policy.discount
                            discount['type'] = 'pct'
                            return discount

            return discount

    def transition_create_service_invoice(self):
        pool = Pool()
        HealthService = pool.get('gnuhealth.health_service')
        Invoice = pool.get('account.invoice')
        Party = pool.get('party.party')
        Journal = pool.get('account.journal')
        AcctConfig = pool.get('account.configuration')
        acct_config = AcctConfig(1)

        currency_id = Transaction().context.get('currency')

        services = HealthService.browse(Transaction().context.get(
            'active_ids'))
        invoices = []

        # Invoice Header
        for service in services:
            if service.state == 'invoiced':
                raise ServiceInvoiced(
                    gettext('health_insurance.msg_service_invoiced')
                )
            if service.invoice_to:
                party = service.invoice_to
            else:
                party = service.patient.party
            invoice_data = {}
            invoice_data['description'] = service.desc
            invoice_data['party'] = party.id
            invoice_data['type'] = 'out'
            invoice_data['invoice_date'] = datetime.date.today()
            invoice_data['company'] = service.company.id

            """ Look for the AR account in the following order:
                * Party
                * Default AR in accounting config
                * Raise an error if there is no AR account
            """
            if (party.account_receivable):
                invoice_data['account'] = party.account_receivable.id
            elif (acct_config.default_account_receivable):
                invoice_data['account'] = \
                    acct_config.default_account_receivable.id
            else:
                raise NoAccountReceivable(
                    gettext('health_insurance.msg_no_account_receivable'))

            ctx = {}
            sale_price_list = None
            if hasattr(party, 'sale_price_list'):
                sale_price_list = party.sale_price_list

            if sale_price_list:
                ctx['price_list'] = sale_price_list.id
                ctx['sale_date'] = datetime.date.today()
                ctx['currency'] = currency_id
                ctx['customer'] = party.id

            journals = Journal.search([
                ('type', '=', 'revenue'),
            ], limit=1)

            if journals:
                journal, = journals
            else:
                journal = None

            invoice_data['journal'] = journal.id

            party_address = Party.address_get(party, type='invoice')
            if not party_address:
                raise NoInvoiceAddress(
                    gettext('health_insurance.msg_no_invoice_address')
                )

            invoice_data['invoice_address'] = party_address.id
            invoice_data['reference'] = service.name

            """ Look for the payment term in the following order:
                * Party
                * Default payment term in accounting config
                * Raise an error if there is no payment term
            """
            if (party.customer_payment_term):
                invoice_data['payment_term'] = party.customer_payment_term.id
            elif (acct_config.default_customer_payment_term):
                invoice_data['payment_term'] = \
                    acct_config.default_customer_payment_term.id
            else:
                raise NoPaymentTerm(
                    gettext('health_insurance.msg_no_payment_term')
                )

            # Invoice Lines
            seq = 0
            invoice_lines = []
            for line in service.service_line:
                seq = seq + 1
                account = line.product.template.account_revenue_used.id

                if sale_price_list:
                    with Transaction().set_context(ctx):
                        unit_price = sale_price_list.compute(
                            party,
                            line.product, line.product.list_price,
                            line.qty, line.product.default_uom)
                else:
                    unit_price = line.product.list_price

                if line.to_invoice:
                    taxes = []
                    desc = line.desc

                    # Include taxes related to the product on the invoice line
                    for product_tax_line in line.product.customer_taxes_used:
                        taxes.append(product_tax_line.id)

                    # Check the Insurance policy for this service
                    if service.insurance_plan:
                        discount = self.discount_policy(
                            service.insurance_plan,
                            line.product)

                        if discount:
                            if 'value' in list(discount.keys()):
                                if discount['value']:
                                    if (discount['type'] == 'pct'):
                                        unit_price *= decimal.Decimal(
                                            1 - discount['value'] / 100)
                                        # Use price_decimal value from
                                        # system configuration to set
                                        # the number of decimals
                                        unit_price = round_price(unit_price)

                                        # Add remark on description discount
                                        str_disc = str(discount['value']) + '%'
                                        desc = line.desc + " (Discnt " + \
                                            str(str_disc) + ")"
                                    # Use the fixed price
                                    else:
                                        unit_price = discount['value']
                                        desc = f"{line.desc} (policy plan)"
                    invoice_lines.append(('create', [{
                        'origin': str(line),
                        'product': line.product.id,
                        'description': desc,
                        'quantity': line.qty,
                        'account': account,
                        'unit': line.product.default_uom.id,
                        'unit_price': unit_price,
                        'sequence': seq,
                        'taxes': [('add', taxes)],
                    }]))
                invoice_data['lines'] = invoice_lines

            invoices.append(invoice_data)

        Invoice.update_taxes(Invoice.create(invoices))

        # Change to invoiced the status on the service document.
        HealthService.write(services, {'state': 'invoiced'})

        return 'end'



# Copyright (C) 2008-2025 Luis Falcon <lfalcon@gnusolidario.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from trytond.model import ModelView, fields
from trytond.pyson import Eval
from trytond.wizard import Wizard
from trytond.pool import Pool

__all__ = ['RequestPatientLabTestStart', 'RequestPatientLabTest']


# Include services in the wizard
class RequestPatientLabTestStart(ModelView):
    'Request Patient Lab Test Start'
    __name__ = 'gnuhealth.patient.lab.test.request.start'

    ungroup_tests = fields.Boolean(
        'Ungroup',
        help="Check if you DO NOT want to include each individual lab test"
        " from this order in the lab test generation step."
        " This is useful when some services are not provided in"
        " the same institution.\n"
        "In this case, you need to individually update the service"
        " document from each individual test")

    service = fields.Many2One(
        'gnuhealth.health_service', 'Service',
        domain=[('patient', '=', Eval('patient'))], depends=['patient'],
        help="Service document associated to this Lab Request")


class RequestPatientLabTest(Wizard):
    'Request Patient Lab Test'
    __name__ = 'gnuhealth.patient.lab.test.request'

    def generate_code(self, **pattern):
        Config = Pool().get('gnuhealth.sequences')
        config = Config(1)
        sequence = config.get_multivalue(
            'lab_request_sequence', **pattern)
        if sequence:
            return sequence.get()

    def append_services(self, labtest, service):
        """ If the ungroup flag is not set, append the lab test
            to the associated health service
        """
        HealthService = Pool().get('gnuhealth.health_service')

        hservice = []

        service_data = {}
        service_lines = []

        # Add the labtest to the service document

        service_lines.append(('create', [{
            'product': labtest.product_id.id,
            'desc': labtest.product_id.rec_name,
            'qty': 1
        }]))

        hservice.append(service)
        service_data['service_line'] = service_lines

        HealthService.write(hservice, service_data)

    def transition_request(self):
        PatientLabTest = Pool().get('gnuhealth.patient.lab.test')
        request_number = self.generate_code()
        lab_tests = []
        for test in self.start.tests:
            lab_test = {}
            lab_test['request'] = request_number
            lab_test['test_type'] = test.id
            lab_test['source_type'] = self.start.source_type
            lab_test['patient_id'] = (self.start.patient
                                      and self.start.patient.id)
            lab_test['other_source'] = self.start.other_source
            lab_test['specimen_type'] = test.specimen_type

            if self.start.doctor:
                lab_test['doctor_id'] = self.start.doctor.id
            if self.start.context:
                lab_test['context'] = self.start.context.id

            lab_test['date'] = self.start.date
            lab_test['urgent'] = self.start.urgent

            if self.start.service:
                lab_test['service'] = self.start.service.id
                # Append the test directly to the health service document
                # if the Ungroup flag is not set (default).
                if not self.start.ungroup_tests:
                    self.append_services(test, self.start.service)
                    lab_test['service_updated'] = 'yes'

            lab_tests.append(lab_test)

        PatientLabTest.create(lab_tests)

        return 'end'



# SPDX-FileCopyrightText: 2009-2013 Bertrand Chenal
# SPDX-FileCopyrightText: 2009-2016 B2CK
# SPDX-FileCopyrightText: 2009-2016 Cédric Krier
# SPDX-FileCopyrightText: 2016-2024 Luis Falcon <falcon@gnuhealth.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from .test_health_caldav import suite

__all__ = ['suite']



# SPDX-FileCopyrightText: 2020-2021 National University of Entre Rios (UNER)
#                         School of Engineering
#                         <saludpublica@ingenieria.uner.edu.ar>
# SPDX-FileCopyrightText: 2020 Mario Puntin <mario@silix.com.ar>
# SPDX-FileCopyrightText: 2020-2024 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2020-2024 GNU Solidario <health@gnusolidario.org>

# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                         HEALTH DENTISTRY package                      #
#         patient_set_odontogram.py: wizard to update odontogram        #
#########################################################################
import json

from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateView, StateTransition, Button
from trytond.pool import Pool
from trytond.transaction import Transaction
from trytond.pyson import Eval, In
from ..health_dentistry import TOOTH_STATE

__all__ = ['SetOdontogramStart', 'SetOdontogram']


permanent = [
    '11', '12', '13', '14', '15', '16', '17', '18',
    '21', '22', '23', '24', '25', '26', '27', '28',
    '31', '32', '33', '34', '35', '36', '37', '38',
    '41', '42', '43', '44', '45', '46', '47', '48'
]

primary = [
    '51', '52', '53', '54', '55',
    '61', '62', '63', '64', '65',
    '71', '72', '73', '74', '75',
    '81', '82', '83', '84', '85'
]

permanent_surfaces = [
    ('vimdp', ['11', '12', '13', '21', '22', '23']),
    ('vomdp', ['14', '15', '16', '17', '18', '24', '25', '26', '27', '28']),
    ('vimdl', ['31', '32', '33', '41', '42', '43']),
    ('vomdl', ['34', '35', '36', '37', '38', '44', '45', '46', '47', '48'])
]

primary_surfaces = [
    ('vimdp', ['51', '52', '53', '61', '62', '63']),
    ('vomdp', ['54', '55', '64', '65']),
    ('vimdl', ['71', '72', '73', '81', '82', '83']),
    ('vomdl', ['74', '75', '84', '85'])
]


def tooth_surface(label, help, tooth):
    return fields.Boolean(label, help=help,
                          states={'readonly': ~In(Eval(tooth), ['D', 'F'])},
                          depends=[tooth])


class SetOdontogramStart(ModelView):
    'Set Odontogram Start'
    __name__ = 'gnuhealth.dentistry.set.odontogram.start'

    include_primary = fields.Boolean('Include Primary')
    t11 = fields.Selection(TOOTH_STATE, '11', sort=False)
    v11 = tooth_surface('V', 'Vestibular', 't11')
    i11 = tooth_surface('I', 'Incisal', 't11')
    m11 = tooth_surface('M', 'Mesial', 't11')
    d11 = tooth_surface('D', 'Distal', 't11')
    p11 = tooth_surface('P', 'Palatine', 't11')
    t12 = fields.Selection(TOOTH_STATE, '12', sort=False)
    v12 = tooth_surface('V', 'Vestibular', 't12')
    i12 = tooth_surface('I', 'Incisal', 't12')
    m12 = tooth_surface('M', 'Mesial', 't12')
    d12 = tooth_surface('D', 'Distal', 't12')
    p12 = tooth_surface('P', 'Palatine', 't12')
    t13 = fields.Selection(TOOTH_STATE, '13', sort=False)
    v13 = tooth_surface('V', 'Vestibular', 't13')
    i13 = tooth_surface('I', 'Incisal', 't13')
    m13 = tooth_surface('M', 'Mesial', 't13')
    d13 = tooth_surface('D', 'Distal', 't13')
    p13 = tooth_surface('P', 'Palatine', 't13')
    t14 = fields.Selection(TOOTH_STATE, '14', sort=False)
    v14 = tooth_surface('V', 'Vestibular', 't14')
    o14 = tooth_surface('O', 'Occlusal', 't14')
    m14 = tooth_surface('M', 'Mesial', 't14')
    d14 = tooth_surface('D', 'Distal', 't14')
    p14 = tooth_surface('P', 'Palatine', 't14')
    t15 = fields.Selection(TOOTH_STATE, '15', sort=False)
    v15 = tooth_surface('V', 'Vestibular', 't15')
    o15 = tooth_surface('O', 'Occlusal', 't15')
    m15 = tooth_surface('M', 'Mesial', 't15')
    d15 = tooth_surface('D', 'Distal', 't15')
    p15 = tooth_surface('P', 'Palatine', 't15')
    t16 = fields.Selection(TOOTH_STATE, '16', sort=False)
    v16 = tooth_surface('V', 'Vestibular', 't16')
    o16 = tooth_surface('O', 'Occlusal', 't16')
    m16 = tooth_surface('M', 'Mesial', 't16')
    d16 = tooth_surface('D', 'Distal', 't16')
    p16 = tooth_surface('P', 'Palatine', 't16')
    t17 = fields.Selection(TOOTH_STATE, '17', sort=False)
    v17 = tooth_surface('V', 'Vestibular', 't17')
    o17 = tooth_surface('O', 'Occlusal', 't17')
    m17 = tooth_surface('M', 'Mesial', 't17')
    d17 = tooth_surface('D', 'Distal', 't17')
    p17 = tooth_surface('P', 'Palatine', 't17')
    t18 = fields.Selection(TOOTH_STATE, '18', sort=False)
    v18 = tooth_surface('V', 'Vestibular', 't18')
    o18 = tooth_surface('O', 'Occlusal', 't18')
    m18 = tooth_surface('M', 'Mesial', 't18')
    d18 = tooth_surface('D', 'Distal', 't18')
    p18 = tooth_surface('P', 'Palatine', 't18')
    t21 = fields.Selection(TOOTH_STATE, '21', sort=False)
    v21 = tooth_surface('V', 'Vestibular', 't21')
    i21 = tooth_surface('I', 'Incisal', 't21')
    m21 = tooth_surface('M', 'Mesial', 't21')
    d21 = tooth_surface('D', 'Distal', 't21')
    p21 = tooth_surface('P', 'Palatine', 't21')
    t22 = fields.Selection(TOOTH_STATE, '22', sort=False)
    v22 = tooth_surface('V', 'Vestibular', 't22')
    i22 = tooth_surface('I', 'Incisal', 't22')
    m22 = tooth_surface('M', 'Mesial', 't22')
    d22 = tooth_surface('D', 'Distal', 't22')
    p22 = tooth_surface('P', 'Palatine', 't22')
    t23 = fields.Selection(TOOTH_STATE, '23', sort=False)
    v23 = tooth_surface('V', 'Vestibular', 't23')
    i23 = tooth_surface('I', 'Incisal', 't23')
    m23 = tooth_surface('M', 'Mesial', 't23')
    d23 = tooth_surface('D', 'Distal', 't23')
    p23 = tooth_surface('P', 'Palatine', 't23')
    t24 = fields.Selection(TOOTH_STATE, '24', sort=False)
    v24 = tooth_surface('V', 'Vestibular', 't24')
    o24 = tooth_surface('O', 'Occlusal', 't24')
    m24 = tooth_surface('M', 'Mesial', 't24')
    d24 = tooth_surface('D', 'Distal', 't24')
    p24 = tooth_surface('P', 'Palatine', 't24')
    t25 = fields.Selection(TOOTH_STATE, '25', sort=False)
    v25 = tooth_surface('V', 'Vestibular', 't25')
    o25 = tooth_surface('O', 'Occlusal', 't25')
    m25 = tooth_surface('M', 'Mesial', 't25')
    d25 = tooth_surface('D', 'Distal', 't25')
    p25 = tooth_surface('P', 'Palatine', 't25')
    t26 = fields.Selection(TOOTH_STATE, '26', sort=False)
    v26 = tooth_surface('V', 'Vestibular', 't26')
    o26 = tooth_surface('O', 'Occlusal', 't26')
    m26 = tooth_surface('M', 'Mesial', 't26')
    d26 = tooth_surface('D', 'Distal', 't26')
    p26 = tooth_surface('P', 'Palatine', 't26')
    t27 = fields.Selection(TOOTH_STATE, '27', sort=False)
    v27 = tooth_surface('V', 'Vestibular', 't27')
    o27 = tooth_surface('O', 'Occlusal', 't27')
    m27 = tooth_surface('M', 'Mesial', 't27')
    d27 = tooth_surface('D', 'Distal', 't27')
    p27 = tooth_surface('P', 'Palatine', 't27')
    t28 = fields.Selection(TOOTH_STATE, '28', sort=False)
    v28 = tooth_surface('V', 'Vestibular', 't28')
    o28 = tooth_surface('O', 'Occlusal', 't28')
    m28 = tooth_surface('M', 'Mesial', 't28')
    d28 = tooth_surface('D', 'Distal', 't28')
    p28 = tooth_surface('P', 'Palatine', 't28')
    t31 = fields.Selection(TOOTH_STATE, '31', sort=False)
    v31 = tooth_surface('V', 'Vestibular', 't31')
    i31 = tooth_surface('I', 'Incisal', 't31')
    m31 = tooth_surface('M', 'Mesial', 't31')
    d31 = tooth_surface('D', 'Distal', 't31')
    l31 = tooth_surface('L', 'Lingual', 't31')
    t32 = fields.Selection(TOOTH_STATE, '32', sort=False)
    v32 = tooth_surface('V', 'Vestibular', 't32')
    i32 = tooth_surface('I', 'Incisal', 't32')
    m32 = tooth_surface('M', 'Mesial', 't32')
    d32 = tooth_surface('D', 'Distal', 't32')
    l32 = tooth_surface('L', 'Lingual', 't32')
    t33 = fields.Selection(TOOTH_STATE, '33', sort=False)
    v33 = tooth_surface('V', 'Vestibular', 't33')
    i33 = tooth_surface('I', 'Incisal', 't33')
    m33 = tooth_surface('M', 'Mesial', 't33')
    d33 = tooth_surface('D', 'Distal', 't33')
    l33 = tooth_surface('L', 'Lingual', 't33')
    t34 = fields.Selection(TOOTH_STATE, '34', sort=False)
    v34 = tooth_surface('V', 'Vestibular', 't34')
    o34 = tooth_surface('O', 'Occlusal', 't34')
    m34 = tooth_surface('M', 'Mesial', 't34')
    d34 = tooth_surface('D', 'Distal', 't34')
    l34 = tooth_surface('L', 'Lingual', 't34')
    t35 = fields.Selection(TOOTH_STATE, '35', sort=False)
    v35 = tooth_surface('V', 'Vestibular', 't35')
    o35 = tooth_surface('O', 'Occlusal', 't35')
    m35 = tooth_surface('M', 'Mesial', 't35')
    d35 = tooth_surface('D', 'Distal', 't35')
    l35 = tooth_surface('L', 'Lingual', 't35')
    t36 = fields.Selection(TOOTH_STATE, '36', sort=False)
    v36 = tooth_surface('V', 'Vestibular', 't36')
    o36 = tooth_surface('O', 'Occlusal', 't36')
    m36 = tooth_surface('M', 'Mesial', 't36')
    d36 = tooth_surface('D', 'Distal', 't36')
    l36 = tooth_surface('L', 'Lingual', 't36')
    t37 = fields.Selection(TOOTH_STATE, '37', sort=False)
    v37 = tooth_surface('V', 'Vestibular', 't37')
    o37 = tooth_surface('O', 'Occlusal', 't37')
    m37 = tooth_surface('M', 'Mesial', 't37')
    d37 = tooth_surface('D', 'Distal', 't37')
    l37 = tooth_surface('L', 'Lingual', 't37')
    t38 = fields.Selection(TOOTH_STATE, '38', sort=False)
    v38 = tooth_surface('V', 'Vestibular', 't38')
    o38 = tooth_surface('O', 'Occlusal', 't38')
    m38 = tooth_surface('M', 'Mesial', 't38')
    d38 = tooth_surface('D', 'Distal', 't38')
    l38 = tooth_surface('L', 'Lingual', 't38')
    t41 = fields.Selection(TOOTH_STATE, '41', sort=False)
    v41 = tooth_surface('V', 'Vestibular', 't41')
    i41 = tooth_surface('I', 'Incisal', 't41')
    m41 = tooth_surface('M', 'Mesial', 't41')
    d41 = tooth_surface('D', 'Distal', 't41')
    l41 = tooth_surface('L', 'Lingual', 't41')
    t42 = fields.Selection(TOOTH_STATE, '42', sort=False)
    v42 = tooth_surface('V', 'Vestibular', 't42')
    i42 = tooth_surface('I', 'Incisal', 't42')
    m42 = tooth_surface('M', 'Mesial', 't42')
    d42 = tooth_surface('D', 'Distal', 't42')
    l42 = tooth_surface('L', 'Lingual', 't42')
    t43 = fields.Selection(TOOTH_STATE, '43', sort=False)
    v43 = tooth_surface('V', 'Vestibular', 't43')
    i43 = tooth_surface('I', 'Incisal', 't43')
    m43 = tooth_surface('M', 'Mesial', 't43')
    d43 = tooth_surface('D', 'Distal', 't43')
    l43 = tooth_surface('L', 'Lingual', 't43')
    t44 = fields.Selection(TOOTH_STATE, '44', sort=False)
    v44 = tooth_surface('V', 'Vestibular', 't44')
    o44 = tooth_surface('O', 'Occlusal', 't44')
    m44 = tooth_surface('M', 'Mesial', 't44')
    d44 = tooth_surface('D', 'Distal', 't44')
    l44 = tooth_surface('L', 'Lingual', 't44')
    t45 = fields.Selection(TOOTH_STATE, '45', sort=False)
    v45 = tooth_surface('V', 'Vestibular', 't45')
    o45 = tooth_surface('O', 'Occlusal', 't45')
    m45 = tooth_surface('M', 'Mesial', 't45')
    d45 = tooth_surface('D', 'Distal', 't45')
    l45 = tooth_surface('L', 'Lingual', 't45')
    t46 = fields.Selection(TOOTH_STATE, '46', sort=False)
    v46 = tooth_surface('V', 'Vestibular', 't46')
    o46 = tooth_surface('O', 'Occlusal', 't46')
    m46 = tooth_surface('M', 'Mesial', 't46')
    d46 = tooth_surface('D', 'Distal', 't46')
    l46 = tooth_surface('L', 'Lingual', 't46')
    t47 = fields.Selection(TOOTH_STATE, '47', sort=False)
    v47 = tooth_surface('V', 'Vestibular', 't47')
    o47 = tooth_surface('O', 'Occlusal', 't47')
    m47 = tooth_surface('M', 'Mesial', 't47')
    d47 = tooth_surface('D', 'Distal', 't47')
    l47 = tooth_surface('L', 'Lingual', 't47')
    t48 = fields.Selection(TOOTH_STATE, '48', sort=False)
    v48 = tooth_surface('V', 'Vestibular', 't48')
    o48 = tooth_surface('O', 'Occlusal', 't48')
    m48 = tooth_surface('M', 'Mesial', 't48')
    d48 = tooth_surface('D', 'Distal', 't48')
    l48 = tooth_surface('L', 'Lingual', 't48')
    t51 = fields.Selection(TOOTH_STATE, '51', sort=False)
    v51 = tooth_surface('V', 'Vestibular', 't51')
    i51 = tooth_surface('I', 'Incisal', 't51')
    m51 = tooth_surface('M', 'Mesial', 't51')
    d51 = tooth_surface('D', 'Distal', 't51')
    p51 = tooth_surface('P', 'Palatine', 't51')
    t52 = fields.Selection(TOOTH_STATE, '52', sort=False)
    v52 = tooth_surface('V', 'Vestibular', 't52')
    i52 = tooth_surface('I', 'Incisal', 't52')
    m52 = tooth_surface('M', 'Mesial', 't52')
    d52 = tooth_surface('D', 'Distal', 't52')
    p52 = tooth_surface('P', 'Palatine', 't52')
    t53 = fields.Selection(TOOTH_STATE, '53', sort=False)
    v53 = tooth_surface('V', 'Vestibular', 't53')
    i53 = tooth_surface('I', 'Incisal', 't53')
    m53 = tooth_surface('M', 'Mesial', 't53')
    d53 = tooth_surface('D', 'Distal', 't53')
    p53 = tooth_surface('P', 'Palatine', 't53')
    t54 = fields.Selection(TOOTH_STATE, '54', sort=False)
    v54 = tooth_surface('V', 'Vestibular', 't54')
    o54 = tooth_surface('O', 'Occlusal', 't54')
    m54 = tooth_surface('M', 'Mesial', 't54')
    d54 = tooth_surface('D', 'Distal', 't54')
    p54 = tooth_surface('P', 'Palatine', 't54')
    t55 = fields.Selection(TOOTH_STATE, '55', sort=False)
    v55 = tooth_surface('V', 'Vestibular', 't55')
    o55 = tooth_surface('O', 'Occlusal', 't55')
    m55 = tooth_surface('M', 'Mesial', 't55')
    d55 = tooth_surface('D', 'Distal', 't55')
    p55 = tooth_surface('P', 'Palatine', 't55')
    t61 = fields.Selection(TOOTH_STATE, '61', sort=False)
    v61 = tooth_surface('V', 'Vestibular', 't61')
    i61 = tooth_surface('I', 'Incisal', 't61')
    m61 = tooth_surface('M', 'Mesial', 't61')
    d61 = tooth_surface('D', 'Distal', 't61')
    p61 = tooth_surface('P', 'Palatine', 't61')
    t62 = fields.Selection(TOOTH_STATE, '62', sort=False)
    v62 = tooth_surface('V', 'Vestibular', 't62')
    i62 = tooth_surface('I', 'Incisal', 't62')
    m62 = tooth_surface('M', 'Mesial', 't62')
    d62 = tooth_surface('D', 'Distal', 't62')
    p62 = tooth_surface('P', 'Palatine', 't62')
    t63 = fields.Selection(TOOTH_STATE, '63', sort=False)
    v63 = tooth_surface('V', 'Vestibular', 't63')
    i63 = tooth_surface('I', 'Incisal', 't63')
    m63 = tooth_surface('M', 'Mesial', 't63')
    d63 = tooth_surface('D', 'Distal', 't63')
    p63 = tooth_surface('P', 'Palatine', 't63')
    t64 = fields.Selection(TOOTH_STATE, '64', sort=False)
    v64 = tooth_surface('V', 'Vestibular', 't64')
    o64 = tooth_surface('O', 'Occlusal', 't64')
    m64 = tooth_surface('M', 'Mesial', 't64')
    d64 = tooth_surface('D', 'Distal', 't64')
    p64 = tooth_surface('P', 'Palatine', 't64')
    t65 = fields.Selection(TOOTH_STATE, '65', sort=False)
    v65 = tooth_surface('V', 'Vestibular', 't65')
    o65 = tooth_surface('O', 'Occlusal', 't65')
    m65 = tooth_surface('M', 'Mesial', 't65')
    d65 = tooth_surface('D', 'Distal', 't65')
    p65 = tooth_surface('P', 'Palatine', 't65')
    t71 = fields.Selection(TOOTH_STATE, '71', sort=False)
    v71 = tooth_surface('V', 'Vestibular', 't71')
    i71 = tooth_surface('I', 'Incisal', 't71')
    m71 = tooth_surface('M', 'Mesial', 't71')
    d71 = tooth_surface('D', 'Distal', 't71')
    l71 = tooth_surface('L', 'Lingual', 't71')
    t72 = fields.Selection(TOOTH_STATE, '72', sort=False)
    v72 = tooth_surface('V', 'Vestibular', 't72')
    i72 = tooth_surface('I', 'Incisal', 't72')
    m72 = tooth_surface('M', 'Mesial', 't72')
    d72 = tooth_surface('D', 'Distal', 't72')
    l72 = tooth_surface('L', 'Lingual', 't72')
    t73 = fields.Selection(TOOTH_STATE, '73', sort=False)
    v73 = tooth_surface('V', 'Vestibular', 't73')
    i73 = tooth_surface('I', 'Incisal', 't73')
    m73 = tooth_surface('M', 'Mesial', 't73')
    d73 = tooth_surface('D', 'Distal', 't73')
    l73 = tooth_surface('L', 'Lingual', 't73')
    t74 = fields.Selection(TOOTH_STATE, '74', sort=False)
    v74 = tooth_surface('V', 'Vestibular', 't74')
    o74 = tooth_surface('O', 'Occlusal', 't74')
    m74 = tooth_surface('M', 'Mesial', 't74')
    d74 = tooth_surface('D', 'Distal', 't74')
    l74 = tooth_surface('L', 'Lingual', 't74')
    t75 = fields.Selection(TOOTH_STATE, '75', sort=False)
    v75 = tooth_surface('V', 'Vestibular', 't75')
    o75 = tooth_surface('O', 'Occlusal', 't75')
    m75 = tooth_surface('M', 'Mesial', 't75')
    d75 = tooth_surface('D', 'Distal', 't75')
    l75 = tooth_surface('L', 'Lingual', 't75')
    t81 = fields.Selection(TOOTH_STATE, '81', sort=False)
    v81 = tooth_surface('V', 'Vestibular', 't81')
    i81 = tooth_surface('I', 'Incisal', 't81')
    m81 = tooth_surface('M', 'Mesial', 't81')
    d81 = tooth_surface('D', 'Distal', 't81')
    l81 = tooth_surface('L', 'Lingual', 't81')
    t82 = fields.Selection(TOOTH_STATE, '82', sort=False)
    v82 = tooth_surface('V', 'Vestibular', 't82')
    i82 = tooth_surface('I', 'Incisal', 't82')
    m82 = tooth_surface('M', 'Mesial', 't82')
    d82 = tooth_surface('D', 'Distal', 't82')
    l82 = tooth_surface('L', 'Lingual', 't82')
    t83 = fields.Selection(TOOTH_STATE, '83', sort=False)
    v83 = tooth_surface('V', 'Vestibular', 't83')
    i83 = tooth_surface('I', 'Incisal', 't83')
    m83 = tooth_surface('M', 'Mesial', 't83')
    d83 = tooth_surface('D', 'Distal', 't83')
    l83 = tooth_surface('L', 'Lingual', 't83')
    t84 = fields.Selection(TOOTH_STATE, '84', sort=False)
    v84 = tooth_surface('V', 'Vestibular', 't84')
    o84 = tooth_surface('O', 'Occlusal', 't84')
    m84 = tooth_surface('M', 'Mesial', 't84')
    d84 = tooth_surface('D', 'Distal', 't84')
    l84 = tooth_surface('L', 'Lingual', 't84')
    t85 = fields.Selection(TOOTH_STATE, '85', sort=False)
    v85 = tooth_surface('V', 'Vestibular', 't85')
    o85 = tooth_surface('O', 'Occlusal', 't85')
    m85 = tooth_surface('M', 'Mesial', 't85')
    d85 = tooth_surface('D', 'Distal', 't85')
    l85 = tooth_surface('L', 'Lingual', 't85')

    @classmethod
    def view_attributes(cls):
        return super(SetOdontogramStart, cls).view_attributes() + [
            ('//page[@id="primary"]', 'states', {
                'invisible': ~Eval('include_primary'),
            })]


class SetOdontogram(Wizard):
    'Set Odontogram'
    __name__ = 'gnuhealth.dentistry.set.odontogram'

    start = StateView('gnuhealth.dentistry.set.odontogram.start',
                      'health_dentistry.set_odontogram_start_view_form',
                      [Button('Cancel', 'end', 'tryton-cancel'),
                       Button('Set', 'set_', 'tryton-ok', default=True)])
    set_ = StateTransition()

    def default_start(self, fields):
        pool = Pool()
        Patient = Pool().get('gnuhealth.patient')
        Treatment = pool.get('gnuhealth.dentistry.treatment')

        patient_id = Transaction().context.get('active_id')
        # Check if wizard called from treatment
        active_model = Transaction().context.get('active_model')
        if active_model == 'gnuhealth.dentistry.treatment':
            patient_id = Treatment.browse(
                [Transaction().context.get('active_id')])[0].patient

        if not patient_id:
            return {}
        res = {
            'include_primary': False,
            't11': '', 't12': '', 't13': '', 't14': '',
            't15': '', 't16': '', 't17': '', 't18': '',
            't21': '', 't22': '', 't23': '', 't24': '',
            't25': '', 't26': '', 't27': '', 't28': '',
            't31': '', 't32': '', 't33': '', 't34': '',
            't35': '', 't36': '', 't37': '', 't38': '',
            't41': '', 't42': '', 't43': '', 't44': '',
            't45': '', 't46': '', 't47': '', 't48': '',
            't51': '', 't52': '', 't53': '', 't54': '', 't55': '',
            't61': '', 't62': '', 't63': '', 't64': '', 't65': '',
            't71': '', 't72': '', 't73': '', 't74': '', 't75': '',
            't81': '', 't82': '', 't83': '', 't84': '', 't85': '',
        }
        patient = Patient.browse([patient_id])
        if patient:
            if patient[0].use_primary_schema:
                res.update({'include_primary': True})
            if patient[0].dental_schema:
                teeth = json.loads(patient[0].dental_schema)
                for t in permanent:
                    res['t' + t] = teeth[t]['ts']
                for ps in permanent_surfaces:
                    for i in range(0, 5):
                        for t in ps[1]:
                            value = teeth[t].get(ps[0][i], False)
                            res[ps[0][i] + t] = True if value else False

            if patient[0].dental_schema_primary:
                teeth = json.loads(patient[0].dental_schema_primary)
                for t in primary:
                    res['t' + t] = teeth[t]['ts']
                for ps in primary_surfaces:
                    for i in range(0, 5):
                        for t in ps[1]:
                            value = teeth[t].get(ps[0][i], False)
                            res[ps[0][i] + t] = True if value else False

        return res

    def transition_set_(self):
        pool = Pool()
        Patient = Pool().get('gnuhealth.patient')
        Treatment = pool.get('gnuhealth.dentistry.treatment')

        patient_id = Transaction().context.get('active_id')
        # Check if wizard called from treatment
        active_model = Transaction().context.get('active_model')
        if active_model == 'gnuhealth.dentistry.treatment':
            patient_id = Treatment.browse(
                [Transaction().context.get('active_id')])[0].patient

        if not patient_id:
            return 'end'

        data = {}
        for t in permanent:
            data[t] = {'ts': getattr(self.start, 't' + t)}
        for ps in permanent_surfaces:
            for i in range(0, 5):
                for t in ps[1]:
                    value = getattr(self.start, ps[0][i] + t, None)
                    if value and data[t]['ts'] in ['D', 'F']:
                        data[t].update({ps[0][i]: 'x'})

        primary_data = {}
        for t in primary:
            primary_data[t] = {'ts': getattr(self.start, 't' + t)}
        for ps in primary_surfaces:
            for i in range(0, 5):
                for t in ps[1]:
                    value = getattr(self.start, ps[0][i] + t, None)
                    if value and primary_data[t]['ts'] in ['D', 'F']:
                        primary_data[t].update({ps[0][i]: 'x'})

        dental_schema = json.dumps(data)
        dental_schema_primary = json.dumps(primary_data)
        patient = Patient(patient_id)
        patient.dental_schema = dental_schema
        patient.dental_schema_primary = dental_schema_primary
        Patient.save([patient])
        return 'end'



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from datetime import datetime
from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateTransition, StateView, Button
from trytond.transaction import Transaction
from trytond.pool import Pool
from trytond.i18n import gettext
from ..exceptions import DestinationBedNotAvailable, ManyRecordsChosen

__all__ = ['CreateBedTransferInit', 'CreateBedTransfer']


class CreateBedTransferInit(ModelView):
    'Create Bed Transfer Init'
    __name__ = 'gnuhealth.bed.transfer.init'
    newbed = fields.Many2One(
        'gnuhealth.hospital.bed', 'New Bed',
        required=True)
    reason = fields.Char('Reason', required=True)

    orig_bed_state = fields.Selection((
        (None, ''),
        ('free', 'Free'),
        ('reserved', 'Reserved'),
        ('occupied', 'Occupied'),
        ('to_clean', 'Needs cleaning'),
        ('na', 'Not available'),
    ), 'Bed of origin Status', sort=False, required=True)

    @staticmethod
    def default_orig_bed_state():
        return 'to_clean'


class CreateBedTransfer(Wizard):
    'Create Bed Transfer'
    __name__ = 'gnuhealth.bed.transfer.create'

    start = StateView(
        'gnuhealth.bed.transfer.init',
        'health_inpatient.view_patient_bed_transfer', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button(
                'Transfer Patient', 'create_bed_transfer', 'tryton-ok',
                True),
        ])
    create_bed_transfer = StateTransition()

    def transition_create_bed_transfer(self):
        inpatient_registrations = Pool(). \
            get('gnuhealth.inpatient.registration')
        bed = Pool().get('gnuhealth.hospital.bed')

        registrations = inpatient_registrations.browse(
            Transaction().context.get('active_ids'))

        # Don't allow mass changes. Work on a single record
        if len(registrations) > 1:
            raise ManyRecordsChosen(
                gettext('health_inpatient.msg_many_records_chosen'))

        registration = registrations[0]
        current_bed = registration.bed
        destination_bed = self.start.newbed
        reason = self.start.reason
        orig_bed_state = self.start.orig_bed_state

        # Check that the new bed is free
        if (destination_bed.state == 'free'):

            # Update bed status with the one given in the transfer
            bed.write([current_bed], {'state': orig_bed_state})

            # Set as occupied the new bed
            bed.write([destination_bed], {'state': 'occupied'})
            # Update the hospitalization record
            hospitalization_info = {}

            hospitalization_info['bed'] = destination_bed

            # Update the hospitalization data
            transfers = []
            transfers.append(('create', [{
                'transfer_date': datetime.now(),
                'bed_from': current_bed,
                'bed_to': destination_bed,
                'reason': reason,
            }]))
            hospitalization_info['bed_transfers'] = transfers

            inpatient_registrations.write([registration], hospitalization_info)

        else:
            raise DestinationBedNotAvailable(
                gettext('health_inpatient.msg_destination_bed_not_available'))

        return 'end'



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from trytond.wizard import Wizard, StateAction
from trytond.transaction import Transaction
from trytond.pool import Pool
from trytond.pyson import PYSONEncoder

from ..exceptions import NoRecordSelected

import gettext

__all__ = ['CreateInpatientEvaluation']


class CreateInpatientEvaluation(Wizard):
    'Create Inpatient Evaluation'
    __name__ = 'wizard.gnuhealth.inpatient.evaluation'

    start_state = 'inpatient_evaluation'
    inpatient_evaluation = StateAction(
        'health_inpatient.act_inpatient_evaluation')

    def do_inpatient_evaluation(self, action):

        inpatient_registration = Transaction().context.get('active_id')

        try:
            reg_id = \
                Pool().get('gnuhealth.inpatient.registration').browse(
                    [inpatient_registration])[0]
        except BaseException:
            raise NoRecordSelected(
                gettext('health_inpatient.msg_no_record_selected'))

        patient = reg_id.patient.id

        action['pyson_domain'] = PYSONEncoder().encode([
            ('patient', '=', patient),
            ('inpatient_registration_code', '=', reg_id.id),
            ('evaluation_type', '=', 'inpatient'),
        ])
        action['pyson_context'] = PYSONEncoder().encode({
            'patient': patient,
            'inpatient_registration_code': reg_id.id,
            'evaluation_type': 'inpatient',
        })

        return action, {}



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2012-2014 Sebastian Marro <smarro@thymbra.com>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from sql import Join, Null
from sql.aggregate import Max, Count
from trytond.model import ModelView, ModelSQL, fields
from trytond.wizard import Wizard, StateView, StateAction, Button
from trytond.pyson import PYSONEncoder
from trytond.pool import Pool
from trytond.transaction import Transaction


__all__ = ['TopDiseases', 'OpenTopDiseasesStart', 'OpenTopDiseases']


class TopDiseases(ModelSQL, ModelView):
    'Top Diseases'
    __name__ = 'gnuhealth.top_diseases'

    disease = fields.Many2One('gnuhealth.pathology', 'Disease')
    cases = fields.Integer('Cases')

    @classmethod
    def __setup__(cls):
        super(TopDiseases, cls).__setup__()
        cls._order.insert(0, ('cases', 'DESC'))

    @staticmethod
    def table_query():
        pool = Pool()
        Evaluation = pool.get('gnuhealth.patient.evaluation')
        evaluation = Evaluation.__table__()
        source = evaluation
        where = evaluation.diagnosis != Null

        if Transaction().context.get('start_date'):
            where &= evaluation.evaluation_start >= \
                Transaction().context['start_date']
        if Transaction().context.get('end_date'):
            where &= evaluation.evaluation_start <= \
                Transaction().context['end_date']
        if Transaction().context.get('group'):
            DiseaseGroupMembers = pool.get('gnuhealth.disease_group.members')
            diseasegroupmembers = DiseaseGroupMembers.__table__()
            join = Join(evaluation, diseasegroupmembers)
            join.condition = join.right.disease == evaluation.diagnosis
            where &= join.right.disease_group == Transaction().context['group']
            source = join

        select = source.select(
            evaluation.diagnosis.as_('id'),
            Max(evaluation.create_uid).as_('create_uid'),
            Max(evaluation.create_date).as_('create_date'),
            Max(evaluation.write_uid).as_('write_uid'),
            Max(evaluation.write_date).as_('write_date'),
            evaluation.diagnosis.as_('disease'),
            Count(evaluation.diagnosis).as_('cases'),
            where=where,
            group_by=evaluation.diagnosis)

        if Transaction().context.get('number_records'):
            select.limit = Transaction().context['number_records']

        return select


class OpenTopDiseasesStart(ModelView):
    'Open Top Diseases'
    __name__ = 'gnuhealth.top_diseases.open.start'

    start_date = fields.Date('Start Date')
    end_date = fields.Date('End Date')
    group = fields.Many2One('gnuhealth.pathology.group', 'Disease Group')
    number_records = fields.Integer('Number of Records', required=True)

    @staticmethod
    def default_number_records():
        return 10


class OpenTopDiseases(Wizard):
    'Open Top Diseases'
    __name__ = 'gnuhealth.top_diseases.open'

    start = StateView(
        'gnuhealth.top_diseases.open.start',
        'health_reporting.top_diseases_open_start_view_form', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Open', 'open_', 'tryton-ok', default=True),
        ])
    open_ = StateAction('health_reporting.act_top_diseases_form')

    def do_open_(self, action):
        action['pyson_context'] = PYSONEncoder().encode({
            'start_date': self.start.start_date,
            'end_date': self.start.end_date,
            'group': self.start.group.id if self.start.group else None,
            'number_records': self.start.number_records,
        })
        return action, {}

    def transition_open_(self):
        return 'end'



# SPDX-FileCopyrightText: 2012-2017 Cédric Krier
# SPDX-FileCopyrightText: 2017-2024 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2017-2024 Luis Falcon <falcon@gnuhealth.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from .test_webdav import suite

__all__ = ['suite']



# Copyright (C) 2008-2025 Luis Falcon <falcon@gnuhealth.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# GNU Health HMIS sequences for this package

from trytond.model import (ModelSQL, ValueMixin, fields)
from trytond.pyson import Id
from trytond.pool import Pool, PoolMeta
# Removed in GH 5.0
# from trytond.tools.multivalue import migrate_property

# Sequences
health_service_sequence = fields.Many2One(
    'ir.sequence', 'Health service sequence', required=True,
    domain=[('sequence_type', '=', Id(
        'health_services', 'seq_type_gnuhealth_health_service'))])


# GNU HEALTH SEQUENCES
class GnuHealthSequences(metaclass=PoolMeta):
    'Standard Sequences for GNU Health'
    __name__ = 'gnuhealth.sequences'

    health_service_sequence = fields.MultiValue(
        health_service_sequence)

    @classmethod
    def default_health_service_sequence(cls, **pattern):
        pool = Pool()
        ModelData = pool.get('ir.model.data')
        try:
            return ModelData.get_id('health_services',
                                    'seq_gnuhealth_health_service')
        except KeyError:
            return None


class _ConfigurationValue(ModelSQL):

    _configuration_value_field = None

    @classmethod
    def __register__(cls, module_name):

        super(_ConfigurationValue, cls).__register__(module_name)


class HealthServiceSequence(_ConfigurationValue, ModelSQL, ValueMixin):
    'Health Service Request Sequence setup'
    __name__ = 'gnuhealth.sequences.health_service_sequence'
    health_service_sequence = health_service_sequence
    _configuration_value_field = 'health_service_sequence'

    @classmethod
    def check_xml_record(cls, records, values):
        return True



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################

from trytond.pool import PoolMeta

__all__ = ['Subdivision']

""" A list of subdivision types not included in pycountry or Tryton
    but that I consider important in our context.
    More info:
    https://en.wikipedia.org/wiki/Administrative_division
"""

SUBDIVISIONS = [
    ('first_nations_reserve', 'First Nations Reserve'),
    ('hamlet', 'Hamlet'),
    ('neighborhood', 'Neighborhood'),
    ('native_american_reserve', 'Native American Reserve'),
    ('resort', 'Resort'),
    ('village', 'Village'),
    ]


class Subdivision(metaclass=PoolMeta):
    __name__ = 'country.subdivision'

    @classmethod
    def __setup__(cls):
        """ Include new subdivisions to Tryton country package
            if they would not exist.
        """
        super().__setup__()
        for subdivision in SUBDIVISIONS:
            if subdivision not in cls.type.selection:
                cls.type.selection.append(subdivision)



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                    HEALTH CRYPTO LAB package                          #
#                health_crypto_lab.py: main module                      #
#########################################################################
from datetime import datetime
from trytond.model import ModelView, fields
from trytond.rpc import RPC
from trytond.pool import PoolMeta
from trytond.pyson import Eval, Not, Bool
import hashlib
import json
from trytond.modules.health.core import get_health_professional

__all__ = ['LabTest']


class LabTest(metaclass=PoolMeta):
    __name__ = 'gnuhealth.lab'

    serializer = fields.Text('Doc String', readonly=True)

    document_digest = fields.Char(
        'Digest', readonly=True,
        help="Original Document Digest")

    digest_status = fields.Function(
        fields.Boolean(
            'Altered',
            help="This field will be set whenever parts of"
            " the main original document has been changed."
            " Please note that the verification is done only on selected"
            " fields."),
        'check_digest')

    serializer_current = fields.Function(
        fields.Text(
            'Current Doc',
            states={
                'invisible': Not(Bool(Eval('digest_status'))),
            }),
        'check_digest')

    digest_current = fields.Function(
        fields.Char(
            'Current Hash',
            states={
                'invisible': Not(Bool(Eval('digest_status'))),
            }),
        'check_digest')

    digital_signature = fields.Text('Digital Signature', readonly=True)

    @classmethod
    def __setup__(cls):
        super(LabTest, cls).__setup__()
        ''' Allow calling the set_signature method via RPC '''
        cls.__rpc__.update({
            'set_signature': RPC(readonly=False),
        })

    @classmethod
    @ModelView.button
    def sign_document(cls, documents):
        document = documents[0]

        # Validate / generate digest for the document
        # and write the name of the signing health professional
        hp = get_health_professional()

        serial_doc = cls.get_serial(document)

        cls.write(documents, {
            'serializer': serial_doc,
            'document_digest': HealthCrypto().gen_hash(serial_doc),
            'validated_by': hp,
            'validation_date': datetime.now(),
            'state': 'validated', })

    @classmethod
    def get_serial(cls, document):

        analyte_line = []

        for line in document.critearea:
            line_elements = [
                line.name or '',
                line.result or '',
                line.units and line.units.name or '',
                line.result_text or '',
                line.remarks or '']

            analyte_line.append(line_elements)

        data_to_serialize = {
            'Lab_test': str(document.name) or '',
            'Test': str(document.test.rec_name) or '',
            'Specimen_type': str(document.specimen_type),
            'HP': (document.requestor
                   and str(document.requestor.rec_name)
                   or ''),
            'Source_type': str(document.source_type),
            'Patient': (document.patient
                        and str(document.patient.rec_name)
                        or ''),
            'Other_source': str(document.other_source) or '',
            'Patient_ID': (document.patient
                           and str(document.patient.party.ref)
                           or ''),
            'Analyte_line': str(analyte_line),
        }

        serialized_doc = str(HealthCrypto().serialize(data_to_serialize))

        return serialized_doc

    @classmethod
    def set_signature(cls, data, signature):
        """
        Set the clearsigned signature
        """

        doc_id = data['id']

        cls.write([cls(doc_id)], {
            'digital_signature': signature,
        })

    def check_digest(self, name):
        result = ''
        serial_doc = self.get_serial(self)
        if (name == 'digest_status' and self.document_digest):
            if (HealthCrypto().gen_hash(serial_doc) == self.document_digest):
                result = False
            else:
                ''' Return true if the document has been altered'''
                result = True
        if (name == 'digest_current'):
            result = HealthCrypto().gen_hash(serial_doc)

        if (name == 'serializer_current'):
            result = serial_doc

        return result

    # Hide the group holding validation information when state is
    # not validated

    @classmethod
    def view_attributes(cls):
        return [('//group[@id="document_digest"]', 'states', {
                'invisible': Not(Eval('state') == 'validated'),
                })]


class HealthCrypto:
    """ GNU Health Cryptographic functions
    """

    def serialize(self, data_to_serialize):
        """ Format to JSON """

        json_output = \
            json.dumps(data_to_serialize, ensure_ascii=False)
        return json_output

    def gen_hash(self, serialized_doc):
        return hashlib.sha512(serialized_doc.encode('utf-8')).hexdigest()



# Copyright (C) 2008-2025 Luis Falcon <lfalcon@gnusolidario.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# Copyright (C) 2013  Sebastian Marro <smarro@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2013 Sebastian Marro <smarro@thymbra.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from trytond.wizard import Wizard, StateView, Button, StateTransition
from trytond.model import ModelView
from trytond.transaction import Transaction
from trytond.pool import Pool
from trytond.i18n import gettext
from trytond.modules.health.core import get_institution_currency
from ..exceptions import (StockMoveExists, NoPharmacy)

__all__ = ['CreatePrescriptionStockMoveInit', 'CreatePrescriptionStockMove']


class CreatePrescriptionStockMoveInit(ModelView):
    'Create Prescription Stock Move Init'
    __name__ = 'gnuhealth.prescription.stock.move.init'


class CreatePrescriptionStockMove(Wizard):
    'Create Prescription Stock Move'
    __name__ = 'gnuhealth.prescription.stock.move.create'

    start = StateView(
        'gnuhealth.prescription.stock.move.init',
        'health_stock.view_create_prescription_stock_move', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button(
                'Create Stock Move', 'create_stock_move',
                'tryton-ok', True),
        ])
    create_stock_move = StateTransition()

    def transition_create_stock_move(self):
        pool = Pool()
        StockMove = pool.get('stock.move')
        Prescription = pool.get('gnuhealth.prescription.order')

        moves = []
        prescriptions = Prescription.browse(Transaction().context.get(
            'active_ids'))
        for prescription in prescriptions:

            if prescription.moves:
                raise StockMoveExists(
                    gettext('health_stock.msg_stock_move_exists')
                )

            if not prescription.pharmacy:
                raise NoPharmacy(
                    gettext('health_stock.msg_no_pharmacy')
                )

            from_location = prescription.pharmacy.warehouse
            if from_location.type == 'warehouse':
                from_location = from_location.storage_location
            to_location = prescription.patient.party.customer_location

            for line in prescription.prescription_line:
                move = StockMove()
                move.origin = prescription
                move.from_location = from_location
                move.to_location = to_location
                move.product = line.medicament.product
                move.unit_price = line.medicament.product.list_price
                move.cost_price = line.medicament.product.cost_price
                move.quantity = line.quantity
                move.unit = line.medicament.product.default_uom
                # Get the Institution (company) currency
                move.currency = get_institution_currency()

                moves.append(move)
        StockMove.save(moves)
        StockMove.do(moves)
        return 'end'



# Copyright (C) 2008-2025 Luis Falcon <lfalcon@gnusolidario.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from trytond.model import ModelView, fields
from trytond.pool import PoolMeta
from trytond.rpc import RPC
from trytond.pyson import Eval, Not, Bool, Equal, Or
import hashlib
import json

__all__ = ['HealthCrypto', 'PatientPrescriptionOrder',
           'BirthCertificate', 'DeathCertificate', 'PatientEvaluation']


class HealthCrypto:
    """ GNU Health Cryptographic functions
    """

    def serialize(self, data_to_serialize):
        """ Format to JSON """
        json_output = \
            json.dumps(data_to_serialize, ensure_ascii=False)
        return json_output

    def gen_hash(self, serialized_doc):
        return str(hashlib.sha512(serialized_doc.encode('utf-8')).hexdigest())


class PatientPrescriptionOrder(metaclass=PoolMeta):
    """Add the serialized and hash fields to the prescription order document"""
    __name__ = 'gnuhealth.prescription.order'

    serializer = fields.Text('Doc String', readonly=True)

    document_digest = fields.Char('Digest', readonly=True,
                                  help="Original Document Digest")

    digest_status = fields.Function(
        fields.Boolean('Altered',
                       states={
                           'invisible': Not(Equal(Eval('state'), 'done')), },
                       help="This field will be set whenever parts of"
                            " the main original document has been changed."
                            " Please note that the verification is done only "
                            "on selected fields."),
        'check_digest')

    serializer_current = fields.Function(
        fields.Text('Current Doc',
                    states={
                        'invisible': Not(Bool(Eval('digest_status'))),
                    }),
        'check_digest')

    digest_current = fields.Function(
        fields.Char('Current Hash',
                    states={
                        'invisible': Not(Bool(Eval('digest_status'))),
                    }),
        'check_digest')

    digital_signature = fields.Text('Digital Signature', readonly=True)

    @staticmethod
    def default_state():
        return 'draft'

    @classmethod
    def __setup__(cls):
        super(PatientPrescriptionOrder, cls).__setup__()
        cls._buttons.update({
            'generate_prescription': {
                'invisible': Equal(Eval('state'), 'validated'),
            },
            'create_prescription': {
                'invisible': Or(Equal(Eval('state'), 'done'),
                                Equal(Eval('state'), 'validated'))
            },

        })
        ''' Allow calling the set_signature method via RPC '''
        cls.__rpc__.update({
            'set_signature': RPC(readonly=False),
        })

    @classmethod
    @ModelView.button
    def generate_prescription(cls, prescriptions):
        prescription = prescriptions[0]

        # Change the state of the prescription to "Validated"
        serial_doc = cls.get_serial(prescription)

        cls.write(prescriptions, {
            'serializer': serial_doc,
            'document_digest': HealthCrypto().gen_hash(serial_doc),
            'state': 'validated', })

    @classmethod
    def get_serial(cls, prescription):

        presc_line = []

        for line in prescription.prescription_line:
            line_elements = [line.medicament and
                             line.medicament.product.name or '',
                             line.dose or '',
                             line.dose_unit and line.dose_unit.name or '',
                             line.route and line.route.name or '',
                             line.form and line.form.name or '',
                             line.indication and line.indication.name or '',
                             line.short_comment or '']

            presc_line.append(line_elements)

        data_to_serialize = {
            'Prescription': str(prescription.prescription_id) or '',
            'Date': str(prescription.prescription_date) or '',
            'HP': str(prescription.healthprof.rec_name),
            'Patient': str(prescription.patient.rec_name),
            'Patient_ID': str(prescription.patient.party.ref) or '',
            'Prescription_line': str(presc_line),
            'Notes': str(prescription.notes),
        }

        serialized_doc = str(HealthCrypto().serialize(data_to_serialize))

        return serialized_doc

    @classmethod
    def set_signature(cls, data, signature):
        """
        Set the clearsigned signature
        """

        doc_id = data['id']

        cls.write([cls(doc_id)], {
            'digital_signature': signature,
        })

    def check_digest(self, name):
        result = ''
        serial_doc = self.get_serial(self)
        if (name == 'digest_status' and self.document_digest):
            if (HealthCrypto().gen_hash(serial_doc) == self.document_digest):
                result = False
            else:
                ''' Return true if the document has been altered'''
                result = True
        if (name == 'digest_current'):
            result = HealthCrypto().gen_hash(serial_doc)

        if (name == 'serializer_current'):
            result = serial_doc

        return result

    # Hide the group holding validation information when state is
    # not validated

    @classmethod
    def view_attributes(cls):
        return [('//group[@id="prescription_digest"]', 'states', {
                'invisible': Not(Eval('state') == 'validated'),
                })]


class BirthCertificate(metaclass=PoolMeta):
    __name__ = 'gnuhealth.birth_certificate'

    serializer = fields.Text('Doc String', readonly=True)

    document_digest = fields.Char('Digest', readonly=True,
                                  help="Original Document Digest")

    digest_status = fields.Function(
        fields.Boolean('Altered',
                       states={
                           'invisible': Not(Equal(Eval('state'), 'done')),
                       },
                       help="This field will be set whenever parts of"
                       " the main original document has been changed."
                       " Please note that the verification is done only"
                       " on selected "
                       " fields."),
        'check_digest')

    serializer_current = fields.Function(
        fields.Text('Current Doc',
                    states={
                        'invisible': Not(Bool(Eval('digest_status'))),
                    }),
        'check_digest')

    digest_current = fields.Function(
        fields.Char('Current Hash',
                    states={
                        'invisible': Not(Bool(Eval('digest_status'))),
                    }),
        'check_digest')

    digital_signature = fields.Text('Digital Signature', readonly=True)

    @classmethod
    def __setup__(cls):
        super(BirthCertificate, cls).__setup__()
        cls._buttons.update({
            'generate_birth_certificate': {
                'invisible': Not(Equal(Eval('state'), 'signed'))},
        })
        ''' Allow calling the set_signature method via RPC '''
        cls.__rpc__.update({
            'set_signature': RPC(readonly=False),
        })

    @classmethod
    @ModelView.button
    def generate_birth_certificate(cls, certificates):
        certificate = certificates[0]

        # Change the state of the certificate to "Done"

        serial_doc = cls.get_serial(certificate)

        cls.write(certificates, {
            'serializer': serial_doc,
            'document_digest': HealthCrypto().gen_hash(serial_doc),
            'state': 'done', })

    @classmethod
    def get_serial(cls, certificate):

        data_to_serialize = {
            'certificate': str(certificate.code) or '',
            'Date': str(certificate.dob) or '',
            'HP': certificate.signed_by
            and str(certificate.signed_by.rec_name) or '',
            'Person': str(certificate.party.rec_name),
            'Person_dob': str(certificate.party.dob) or '',
            'Person_ID': str(certificate.party.ref) or '',
            # country.rec_name has flag emoji, which will not show
            # properly when no proper fonts is installed.
            'Country': str(certificate.country.name) or '',
            'Country_subdivision': certificate.country_subdivision
            and str(certificate.country_subdivision.rec_name) or '',
            'Mother': certificate.mother
            and str(certificate.mother.rec_name) or '',
            'Father': certificate.father
            and str(certificate.father.rec_name) or '',
            'Observations': str(certificate.observations),
        }

        serialized_doc = str(HealthCrypto().serialize(data_to_serialize))

        return serialized_doc

    @classmethod
    def set_signature(cls, data, signature):
        """
        Set the clearsigned signature
        """
        doc_id = data['id']

        cls.write([cls(doc_id)], {
            'digital_signature': signature,
        })

    def check_digest(self, name):
        result = ''
        serial_doc = self.get_serial(self)
        if (name == 'digest_status' and self.document_digest):
            if (HealthCrypto().gen_hash(serial_doc) == self.document_digest):
                result = False
            else:
                ''' Return true if the document has been altered'''
                result = True
        if (name == 'digest_current'):
            result = HealthCrypto().gen_hash(serial_doc)

        if (name == 'serializer_current'):
            result = serial_doc

        return result

    # Hide the group holding all the digital signature until signed

    @classmethod
    def view_attributes(cls):
        return [('//group[@id="group_current_string"]', 'states', {
                'invisible': ~Eval('digest_status'),
                })]


class DeathCertificate(metaclass=PoolMeta):

    __name__ = 'gnuhealth.death_certificate'

    serializer = fields.Text('Doc String', readonly=True)

    document_digest = fields.Char('Digest', readonly=True,
                                  help="Original Document Digest")

    digest_status = fields.Function(
        fields.Boolean('Altered',
                       states={
                           'invisible': Not(Equal(Eval('state'), 'done')),
                       },
                       help="This field will be set whenever parts of"
                       " the main original document has been changed."
                       " Please note that the verification is done "
                       " only on selected fields."),
        'check_digest')

    serializer_current = fields.Function(
        fields.Text('Current Doc',
                    states={
                        'invisible': Not(Bool(Eval('digest_status'))),
                    }),
        'check_digest')

    digest_current = fields.Function(
        fields.Char('Current Hash',
                    states={
                        'invisible': Not(Bool(Eval('digest_status'))),
                    }),
        'check_digest')

    digital_signature = fields.Text('Digital Signature', readonly=True)

    @classmethod
    def __setup__(cls):
        super(DeathCertificate, cls).__setup__()
        cls._buttons.update({
            'generate_death_certificate': {
                'invisible': Not(Equal(Eval('state'), 'signed')),
            },
        })
        ''' Allow calling the set_signature method via RPC '''
        cls.__rpc__.update({
            'set_signature': RPC(readonly=False),
        })

    @classmethod
    @ModelView.button
    def generate_death_certificate(cls, certificates):
        certificate = certificates[0]

        # Change the state of the certificate to "Done"

        serial_doc = cls.get_serial(certificate)

        cls.write(certificates, {
            'serializer': serial_doc,
            'document_digest': HealthCrypto().gen_hash(serial_doc),
            'state': 'done', })

    @classmethod
    def get_serial(cls, certificate):

        underlying_conds = []

        for condition in certificate.underlying_conditions:
            cond = []
            cond = [str(condition.condition.rec_name),
                    condition.interval,
                    condition.unit_of_time]

            underlying_conds.append(cond)

        data_to_serialize = {
            'certificate': str(certificate.code) or '',
            'Date': str(certificate.dod) or '',
            'HP': certificate.signed_by
            and str(certificate.signed_by.rec_name) or '',
            'Person': str(certificate.party.rec_name),
            'Person_dob': str(certificate.party.dob) or '',
            'Person_ID': str(certificate.party.ref) or '',
            'Cod': str(certificate.cod.rec_name),
            'Underlying_conditions': underlying_conds or '',
            'Autopsy': certificate.autopsy,
            'Type_of_death': str(certificate.type_of_death),
            'Place_of_death': str(certificate.place_of_death),
            # country.rec_name has flag emoji, which will not show
            # properly when no proper fonts is installed.
            'Country': str(certificate.country.name) or '',
            'Country_subdivision': certificate.country_subdivision
            and str(certificate.country_subdivision.rec_name) or '',
            'Observations': str(certificate.observations),
        }

        serialized_doc = str(HealthCrypto().serialize(data_to_serialize))

        return serialized_doc

    @classmethod
    def set_signature(cls, data, signature):
        """
        Set the clearsigned signature
        """
        doc_id = data['id']

        cls.write([cls(doc_id)], {
            'digital_signature': signature,
        })

    def check_digest(self, name):
        result = ''
        serial_doc = self.get_serial(self)
        if (name == 'digest_status' and self.document_digest):
            if (HealthCrypto().gen_hash(serial_doc) == self.document_digest):
                result = False
            else:
                ''' Return true if the document has been altered'''
                result = True
        if (name == 'digest_current'):
            result = HealthCrypto().gen_hash(serial_doc)

        if (name == 'serializer_current'):
            result = serial_doc

        return result

    # Hide the group holding all the digital signature until signed

    @classmethod
    def view_attributes(cls):
        return [('//group[@id="group_current_string"]', 'states', {
                'invisible': ~Eval('digest_status'),
                })]


class PatientEvaluation(metaclass=PoolMeta):
    __name__ = 'gnuhealth.patient.evaluation'

    serializer = fields.Text('Doc String', readonly=True)

    document_digest = fields.Char('Digest', readonly=True,
                                  help="Original Document Digest")

    digest_status = fields.Function(
        fields.Boolean('Altered',
                       states={
                           'invisible': Not(Equal(Eval('state'), 'signed')),
                       },
                       help="This field will be set whenever parts of"
                            " the main original document has been changed."
                            " Please note that the verification is done"
                            " only on selected fields."),
        'check_digest')

    serializer_current = fields.Function(
        fields.Text('Current Doc',
                    states={
                        'invisible': Not(Bool(Eval('digest_status'))),
                    }),
        'check_digest')

    digest_current = fields.Function(
        fields.Char('Current Hash',
                    states={
                        'invisible': Not(Bool(Eval('digest_status'))),
                    }),
        'check_digest')

    digital_signature = fields.Text('Digital Signature', readonly=True)

    @classmethod
    def __setup__(cls):
        super(PatientEvaluation, cls).__setup__()
        cls._buttons.update({
            'sign_evaluation': {
                'invisible': Not(Equal(Eval('state'), 'done')),
            },
        })
        ''' Allow calling the set_signature method via RPC '''
        cls.__rpc__.update({
            'set_signature': RPC(readonly=False),
        })

    @classmethod
    @ModelView.button
    def sign_evaluation(cls, evaluations):
        evaluation = evaluations[0]

        # Change the state of the evaluation to "Signed"

        serial_doc = cls.get_serial(evaluation)

        cls.write(evaluations, {
            'serializer': serial_doc,
            'document_digest': HealthCrypto().gen_hash(serial_doc),
            'state': 'signed', })

    @classmethod
    def get_serial(cls, evaluation):

        signs_symptoms = []
        secondary_conditions = []
        diagnostic_hypotheses = []
        procedures = []

        for sign_symptom in evaluation.signs_and_symptoms:
            finding = []
            finding = [sign_symptom.clinical.rec_name,
                       sign_symptom.sign_or_symptom,
                       ]

            signs_symptoms.append(finding)

        for secondary_condition in evaluation.secondary_conditions:
            sc = []
            sc = [secondary_condition.pathology.rec_name]

            secondary_conditions.append(sc)

        for ddx in evaluation.diagnostic_hypothesis:
            dx = []
            dx = [ddx.pathology.rec_name]

            diagnostic_hypotheses.append(dx)

        for procedure in evaluation.actions:
            proc = []
            proc = [procedure.procedure.rec_name]

            procedures.append(proc)

        data_to_serialize = {
            'Patient': str(evaluation.patient.rec_name) or '',
            'Start': str(evaluation.evaluation_start) or '',
            'End': str(evaluation.evaluation_endtime) or '',
            'Initiated_by': str(evaluation.healthprof.rec_name),
            'Signed_by': evaluation.signed_by and
            str(evaluation.signed_by.rec_name) or '',
            'Specialty': evaluation.specialty and
            str(evaluation.specialty.rec_name) or '',
            'Visit_type': str(evaluation.visit_type) or '',
            'Urgency': str(evaluation.urgency) or '',
            'Information_source': str(evaluation.information_source) or '',
            'Reliable_info': evaluation.reliable_info,
            'Chief_complaint': str(evaluation.chief_complaint) or '',
            'Present_illness': str(evaluation.present_illness) or '',
            'Evaluation_summary': str(evaluation.evaluation_summary),
            'Signs_and_Symptoms': signs_symptoms or '',
            'Glycemia': evaluation.glycemia or '',
            'Hba1c': evaluation.hba1c or '',
            'Total_Cholesterol': evaluation.cholesterol_total or '',
            'HDL': evaluation.hdl or '',
            'LDL': evaluation.ldl or '',
            'TAG': evaluation.ldl or '',
            'Systolic': evaluation.systolic or '',
            'Diastolic': evaluation.diastolic or '',
            'BPM': evaluation.bpm or '',
            'Respiratory_rate': evaluation.respiratory_rate or '',
            'Osat': evaluation.osat or '',
            'Malnutrition': evaluation.malnutrition,
            'Dehydration': evaluation.dehydration,
            'Temperature': evaluation.temperature,
            'Weight': evaluation.weight or '',
            'Height': evaluation.height or '',
            'BMI': evaluation.bmi or '',
            'Head_circ': evaluation.head_circumference or '',
            'Abdominal_cir': evaluation.abdominal_circ or '',
            'Hip': evaluation.hip or '',
            'WHR': evaluation.whr or '',
            'Loc': evaluation.loc or '',
            'Loc_eyes': evaluation.loc_eyes or '',
            'Loc_verbal': evaluation.loc_verbal or '',
            'Loc_motor': evaluation.loc_motor or '',
            'Tremor': evaluation.tremor,
            'Violent': evaluation.violent,
            'Mood': str(evaluation.mood) or '',
            'Orientation': evaluation.orientation,
            'Memory': evaluation.memory,
            'Knowledge_current_events': evaluation.knowledge_current_events,
            'Judgment': evaluation.judgment,
            'Abstraction': evaluation.abstraction,
            'Vocabulary': evaluation.vocabulary,
            'Calculation': evaluation.calculation_ability,
            'Object_recognition': evaluation.object_recognition,
            'Praxis': evaluation.praxis,
            'Diagnosis': evaluation.diagnosis and
            str(evaluation.diagnosis.rec_name) or '',
            'Secondary_conditions': secondary_conditions or '',
            'DDX': diagnostic_hypotheses or '',
            'Info_Diagnosis': str(evaluation.info_diagnosis) or '',
            'Treatment_plan': str(evaluation.directions) or '',
            'Procedures': procedures or '',
            'Institution': evaluation.institution and
            str(evaluation.institution.rec_name) or '',
            'Derived_from': evaluation.derived_from and
            str(evaluation.derived_from.rec_name) or '',
            'Derived_to': evaluation.derived_to and
            str(evaluation.derived_to.rec_name) or '',
        }

        serialized_doc = str(HealthCrypto().serialize(data_to_serialize))

        return serialized_doc

    @classmethod
    def set_signature(cls, data, signature):
        """
        Set the clearsigned signature
        """
        doc_id = data['id']

        cls.write([cls(doc_id)], {
            'digital_signature': signature,
        })

    def check_digest(self, name):
        result = ''
        serial_doc = str(self.get_serial(self))
        if (name == 'digest_status' and self.document_digest):
            if (HealthCrypto().gen_hash(serial_doc) == self.document_digest):
                result = False
            else:
                ''' Return true if the document has been altered'''
                result = True
        if (name == 'digest_current'):
            result = HealthCrypto().gen_hash(serial_doc)

        if (name == 'serializer_current'):
            result = serial_doc

        return result
    # Hide the group holding all the digital signature until signed

    @classmethod
    def view_attributes(cls):
        return [('//group[@id="group_digital_signature"]', 'states', {
                'invisible': ~Eval('digital_signature')}),
                ('//group[@id="group_current_string"]', 'states', {
                 'invisible': ~Eval('digest_status'),
                 })]



# Copyright (C) 2008-2025 Luis Falcon <falcon@gnuhealth.org>
# Copyright (C) 2011-2025 GNU Solidario <health@gnusolidario.org>
# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# GNU Health HMIS sequences for this package

from trytond.model import (ModelSQL, ValueMixin, fields)
from trytond.pyson import Id
from trytond.pool import Pool, PoolMeta
# Removed in GH 5.0
# from trytond.tools.multivalue import migrate_property

# Sequences
inpatient_registration_sequence = fields.Many2One(
    'ir.sequence', 'Inpatient Registration Sequence', required=True,
    domain=[('sequence_type', '=', Id(
        'health_inpatient', 'seq_type_gnuhealth_inpatient_registration'))])

inpatient_meal_order_sequence = fields.Many2One(
    'ir.sequence', 'Inpatient Meal order Sequence', required=True,
    domain=[('sequence_type', '=', Id(
        'health_inpatient', 'seq_type_gnuhealth_inpatient_meal_order'))])


patient_rounding_sequence = fields.Many2One(
    'ir.sequence', 'Patient Rounding Sequence', required=True,
    domain=[('sequence_type', '=', Id(
        'health_inpatient', 'seq_type_gnuhealth_patient_rounding'))])


# GNU HEALTH SEQUENCES
class GnuHealthSequences(metaclass=PoolMeta):
    'Standard Sequences for GNU Health'
    __name__ = 'gnuhealth.sequences'

    inpatient_registration_sequence = fields.MultiValue(
        inpatient_registration_sequence)

    inpatient_meal_order_sequence = fields.MultiValue(
        inpatient_meal_order_sequence)

    patient_rounding_sequence = fields.MultiValue(
        patient_rounding_sequence)

    @classmethod
    def default_inpatient_registration_sequence(cls, **pattern):
        pool = Pool()
        ModelData = pool.get('ir.model.data')
        try:
            return ModelData.get_id('health_inpatient',
                                    'seq_gnuhealth_inpatient_registration')
        except KeyError:
            return None

    @classmethod
    def default_inpatient_meal_order_sequence(cls, **pattern):
        pool = Pool()
        ModelData = pool.get('ir.model.data')
        try:
            return ModelData.get_id('health_inpatient',
                                    'seq_gnuhealth_inpatient_meal_order')
        except KeyError:
            return None

    @classmethod
    def default_patient_rounding_sequence(cls, **pattern):
        pool = Pool()
        ModelData = pool.get('ir.model.data')
        try:
            return ModelData.get_id('health_inpatient',
                                    'seq_gnuhealth_patient_rounding')
        except KeyError:
            return None


class _ConfigurationValue(ModelSQL):

    _configuration_value_field = None

    @classmethod
    def __register__(cls, module_name):
        super(_ConfigurationValue, cls).__register__(module_name)

    """
    @classmethod
    def _migrate_property(cls, field_names, value_names, fields):
        field_names.append(cls._configuration_value_field)
        value_names.append(cls._configuration_value_field)
        migrate_property(
            'gnuhealth.sequences', field_names, cls, value_names,
            fields=fields)

    """


class InpatientRegistrationSequence(_ConfigurationValue, ModelSQL, ValueMixin):
    'Inpatient Care Sequences setup'
    __name__ = 'gnuhealth.sequences.inpatient_registration_sequence'
    inpatient_registration_sequence = inpatient_registration_sequence
    _configuration_value_field = 'inpatient_registration_sequence'

    @classmethod
    def check_xml_record(cls, records, values):
        return True


class InpatientMealOrderSequence(_ConfigurationValue, ModelSQL, ValueMixin):
    'Patient Evaluation Sequence setup'
    __name__ = 'gnuhealth.sequences.inpatient_meal_order_sequence'
    inpatient_meal_order_sequence = inpatient_meal_order_sequence
    _configuration_value_field = 'inpatient_meal_order_sequence'

    @classmethod
    def check_xml_record(cls, records, values):
        return True


class PatientRoundingSequence(_ConfigurationValue, ModelSQL, ValueMixin):
    'Patient Evaluation Sequence setup'
    __name__ = 'gnuhealth.sequences.patient_rounding_sequence'
    patient_rounding_sequence = patient_rounding_sequence
    _configuration_value_field = 'patient_rounding_sequence'

    @classmethod
    def check_xml_record(cls, records, values):
        return True



# SPDX-FileCopyrightText: 2019-2022 Chris Zimmerman <chris@teffalump.com>
# SPDX-FileCopyrightText: 2021-2024 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2023 Patryk Rosik <p.rosik@stud.uni-hannover.de>
# SPDX-FileCopyrightText: 2023 Feng Shu <tumashu@163.com>
# SPDX-FileCopyrightText: 2021-2024 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                         HEALTH IMAGING WORKLIST package               #
#                     health_imaging_worklist.py: main module           #
#########################################################################

"""
Worklist feature of health_imaging.

This module let health_imaging support worklist feature.
"""

from trytond.model import ModelView, ModelSQL, fields
from trytond.pyson import Eval, Not, Bool
from trytond.pool import Pool, PoolMeta
from trytond.modules.health.core import (get_institution,
                                         compute_age_from_dates,
                                         parse_compute_age)

from genshi.template import NewTextTemplate
from pydicom.uid import generate_uid

import logging
import json

__all__ = [
    "WorklistTemplate",
    "ImagingTestRequest",
    "ImagingTest",
    "TestResult",
]

logger = logging.getLogger(__name__)

# XXX: Maybe we should find a better org root string for
# gnuhealth, or let org root string configable.
gnuhealth_org_root = '1.2.836.0.1.3240043.7.198.'


class WorklistTemplate(ModelSQL, ModelView):
    """Worklist Template"""
    __name__ = "gnuhealth.imaging_worklist.worklist_template"
    _rec_name = "name"

    name = fields.Char(
        "Name", required=True,
        help="Worklist template name")

    template_type = fields.Selection([
        ('dump2dcm', 'dump2dcm'),
        ('json', 'json'),
    ], 'Template Type', sort=False)

    template = fields.Text(
        "Template", required=True,
        help="Genshi syntax template used to create worklist text, "
        "if template type is dump2dcm, worklist wl file can be generated "
        "by dump2dcm command of dcmtk. "
        "if template type is json, worklist wl file can be generated "
        "from json by tool like python-orthanc-tools.")

    charset = fields.Char(
        'Charset',
        help='This field is used to store SpecificCharacterSet tag '
        '(0008,0005) of worklist, for example: ISO_IR 100, ISO_IR 192,'
        ' GBK ...')

    comment = fields.Text('Comment')

    @staticmethod
    def default_charset():
        return 'IS0_IR 192'

    @staticmethod
    def default_template_type():
        return 'dump2dcm'

    @classmethod
    def default_template(cls, **abc):
        template = """\
{% if template_type=='dump2dcm' %}\\
(0008,0005) SH [$SpecificCharacterSet]
(0008,0201) SH [$TimezoneOffsetFromUTC]
(0008,0050) SH [$AccessionNumber]
(0040,1001) SH [$RequestedProcedureID]
(0020,000d) UI [$StudyInstanceUID]
(0010,0010) PN [$PatientName]
(0010,0020) LO [$PatientID]
(0010,1010) AS [$PatientAge]
(0010,0030) DA [$PatientBirthDate]
(0010,0040) CS [$PatientSex]
(0032,1032) PN [$RequestingPhysician]
(0032,1033) LO [$RequestingService]
(0008,0090) PN [$ReferringPhysicianName]
(0008,0080) LO [$InstitutionName]
(0032,1060) LO [$RequestedProcedureDescription]
(0040,0100) SQ (Sequence with undefined length)
  (fffe,e000) na (Item with undefined length)
    (0008,0060) CS [$Modality]
    (0040,0001) AE [$ScheduledStationAETitle]
    (0040,0002) DA [$ScheduledProcedureStepStartDate]
    (0040,0003) TM [$ScheduledProcedureStepStartTime]
  (fffe,e00d) na (ItemDelimitationItem)
(fffe,e0dd) na (SequenceDelimitationItem)
{% end %}\\
\\
{% if template_type=='json' %}\\
{
    "SpecificCharacterSet": "$SpecificCharacterSet",
    "TimezoneOffsetFromUTC": "$TimezoneOffsetFromUTC",
    "AccessionNumber": "$AccessionNumber",
    "RequestedProcedureID": "$RequestedProcedureID",
    "StudyInstanceUID": "$StudyInstanceUID",
    "PatientName": "$PatientName",
    "PatientID": "$PatientID",
    "PatientAge": "$PatientAge",
    "PatientBirthDate": "$PatientBirthDate",
    "PatientSex": "$PatientSex",
    "RequestingPhysician": "$RequestingPhysician",
    "RequestingService": "$RequestingService",
    "ReferringPhysicianName": "$ReferringPhysicianName",
    "InstitutionName": "$InstitutionName",
    "RequestedProcedureDescription": "$RequestedProcedureDescription",
    "Modality": "$Modality",
    "ScheduledStationAETitle": "$ScheduledStationAETitle",
    "ScheduledProcedureStepStartDate": "$ScheduledProcedureStepStartDate",
    "ScheduledProcedureStepStartTime": "$ScheduledProcedureStepStartTime",
    "ScheduledProcedureStepID": "$ScheduledProcedureStepID"
}
{% end %}\
"""
        return template


class ImagingTestRequest(metaclass=PoolMeta):
    __name__ = 'gnuhealth.imaging.test.request'

    computed_age = fields.Function(fields.Char(
        'Age',
        help="Computed patient age at image request."),
        'patient_age_at_imaging_request')

    def patient_age_at_imaging_request(self, name):
        if (self.patient.party.dob and self.date):
            return compute_age_from_dates(
                self.patient.party.dob, None, None, None, 'age',
                self.date.date())

    merge_id = fields.Char("Merge ID")

    @staticmethod
    def default_merge_id():
        # Use DICOM UID format, for most situation, merge id is used
        # as StudyInstanceUID.
        return generate_uid(gnuhealth_org_root)

    show_worklist_text = fields.Boolean('Worklist Preview')

    @staticmethod
    def default_show_worklist_text():
        return False

    worklist_status = fields.Selection([
        ('todo', 'Todo'),
        ('done', 'Done'),
        ('canceled', 'Canceled'),
    ], 'Worklist Status', sort=False,
        help="Worklist processing status.")

    @staticmethod
    def default_worklist_status():
        return 'todo'

    worklist_text = fields.Function(
        fields.Text("Worklist text",
                    states={'invisible': Not(
                        Bool(Eval('show_worklist_text')))}),
        'get_worklist_text')

    def get_worklist_text(self, name):
        template = self.get_worklist_template()
        template_type = self.get_worklist_template_type()
        if template:
            data = self.get_worklist_template_data()
            data = {k: self.quote_template_value(v, template_type)
                    for (k, v) in data.items()}
            tmpl = NewTextTemplate(template)
            text = str(tmpl.generate(**data))
            return text
        else:
            return ''

    @classmethod
    def quote_template_value(cls, value, template_type):
        if isinstance(value, str):
            if template_type == 'json':
                value = json.dumps(value, ensure_ascii=False)[1:][:-1]
            elif template_type == 'dump2dcm':
                value = value.replace('\n', ' ')

        return value

    def get_worklist_template(self):
        template = (
            self.requested_test.imaging_worklist_template and
            self.requested_test.imaging_worklist_template.template)
        return template

    def get_worklist_template_type(self):
        template_type = (
            self.requested_test.imaging_worklist_template and
            self.requested_test.imaging_worklist_template.template_type)
        return template_type

    def get_worklist_template_data(self):
        data = {
            # We can not use 'self' as key name, so use 'my'
            # instead.
            'my': self,
            'template_type': self.get_worklist_template_type(),
            'MergeID': self.merge_id or '',
            'SpecificCharacterSet': self.getDicomSpecificCharacterSet(),
            'AccessionNumber': self.getDicomAccessionNumber(),
            'RequestedProcedureID': self.getDicomRequestedProcedureID(),
            'StudyInstanceUID': self.getDicomStudyInstanceUID(),
            'PatientName': self.getDicomPatientName(),
            'PatientID': self.getDicomPatientID(),
            'PatientAge': self.getDicomPatientAge(),
            'PatientBirthDate': self.getDicomPatientBirthDate(),
            'PatientSex': self.getDicomPatientSex(),
            'RequestingPhysician': self.getDicomRequestingPhysician(),
            'RequestingService': self.getDicomRequestingService(),
            'InstitutionName': self.getDicomInstitutionName(),
            'Modality': self.getDicomModality(),
            'ReferringPhysicianName':
            self.getDicomReferringPhysicianName(),
            'RequestedProcedureDescription':
            self.getDicomRequestedProcedureDescription(),
            'ScheduledStationAETitle':
            self.getDicomScheduledStationAETitle(),
            'ScheduledProcedureStepStartDate':
            self.getDicomScheduledProcedureStepStartDate(),
            'ScheduledProcedureStepStartTime':
            self.getDicomScheduledProcedureStepStartTime(),
            'ScheduledProcedureStepID':
            self.getDicomScheduledProcedureStepID(),
            'TimezoneOffsetFromUTC':
            self.getDicomTimezoneOffsetFromUTC(),
        }
        return data

    def getDicomSpecificCharacterSet(self):
        charset = (
            self.requested_test.imaging_worklist_template and
            self.requested_test.imaging_worklist_template.charset)
        return charset

    def getDicomAccessionNumber(self):
        return self.request or ''

    def getDicomRequestedProcedureID(self):
        return self.request_line or ''

    def getDicomStudyInstanceUID(self):
        return self.merge_id or ''

    def getDicomPatientName(self):
        name = (self.format_dicom_person_name(self.patient.party.id)
                or (self.patient and self.patient.rec_name) or '')
        return name

    def format_dicom_person_name(self, person_id):
        Pname = Pool().get('gnuhealth.person_name')

        try:
            officialname = Pname.search(
                [("party", "=", person_id), ("use", "=", 'official')])[0]
        except BaseException:
            officialname = None

        if officialname:
            family = officialname.family or ''
            given = officialname.given or ''
            # gnuhealth.person_name do not support middle name.
            middle = ''
            prefix = officialname.prefix or ''
            suffix = officialname.suffix or ''
            name = "^".join([
                family, given, middle, prefix, suffix]).rstrip('^')
            return name

    def getDicomPatientID(self):
        return self.patient and self.patient.puid or ''

    def getDicomPatientBirthDate(self):
        dob = self.patient and self.patient.party.dob
        if dob:
            return dob.strftime('%Y%m%d')

    def getDicomPatientAge(self):
        age_str = self.computed_age
        if age_str:
            year, month, day = parse_compute_age(age_str)

            # Handle y, m, d = None
            if year is None:
                year = -1
            if month is None:
                month = -1
            if day is None:
                day = -1

            if year == 0 and month == 0 and day > 0:
                return f'{day:03}D'
            elif year == 0 and month > 0:
                return f'{month:03}M'
            elif year > 0:
                return f'{year:03}Y'
            else:
                return ''

    def getDicomPatientSex(self):
        sex = self.patient and self.patient.gender
        if sex == 'f':
            return 'F'
        elif sex == 'm':
            return 'M'
        else:
            return "O"

    def getDicomRequestingPhysician(self):
        """
        Returns the health professional who requests  the test
        """
        name = (self.format_dicom_person_name(self.doctor.party.id)
                or (self.doctor and self.doctor.rec_name) or '')
        return name

    def getDicomRequestingService(self):
        """
        Returns the specialty of the physician associated to the test
        as the Service
        """
        name = ''
        if (self.doctor.main_specialty):
            name = self.doctor.main_specialty.rec_name
        return name

    def getDicomReferringPhysicianName(self):
        """
        Returns the health professional who sent / derived the patient
        to this unit
        """

        name = (self.format_dicom_person_name(self.doctor.party.id)
                or (self.doctor and self.doctor.rec_name) or '')
        return name

    def getDicomInstitutionName(self):
        # Return the name (string) of the institution
        institution_id = get_institution()
        if institution_id:
            institution = \
                Pool().get('gnuhealth.institution')(institution_id)
            return institution.rec_name
        else:
            return ''

    def getDicomRequestedProcedureDescription(self):
        test = self.requested_test and self.requested_test.rec_name or ''
        return test

    def getDicomScheduledStationAETitle(self):
        aetitle = self.requested_test.aetitle or ''
        return aetitle

    def getDicomScheduledProcedureStepStartDate(self):
        # This is UTC datetime, so we need set dicom tag (0008,0201)
        # 'Timezone Offset From UTC' to '+0000'.
        date = self.date.strftime('%Y%m%d')
        return date

    def getDicomScheduledProcedureStepStartTime(self):
        # This is UTC datetime, so we need set dicom tag (0008,0201)
        # 'Timezone Offset From UTC' to '+0000'.
        time = self.date.strftime('%H%M%S')
        return time

    def getDicomScheduledProcedureStepID(self):
        # FIXME: how to get proper value of this tag from gnuhealth?
        # request number is enough?
        return self.request or ''

    def getDicomTimezoneOffsetFromUTC(self):
        # Datetimes get from gnuhealth are UTC datetimes, so we need
        # set dicom tag (0008,0201) 'Timezone Offset From UTC' to
        # '+0000'.
        return '+0000'

    def getDicomModality(self):
        test_type = (self.requested_test.test_type and
                     self.requested_test.test_type.code or '')
        return test_type


class ImagingTest(metaclass=PoolMeta):
    __name__ = 'gnuhealth.imaging.test'

    aetitle = fields.Char(
        "AETitle",
        help="AETitle string, used as (0040,0001) "
        "ScheduledStationAETitle tag in worklist template."
    )

    # NOTE: health_orthanc has 'worklist_template' field, so we use
    # 'imaging_worklist_template' instead, which let database upgrade
    # a bit easier.
    imaging_worklist_template = fields.Many2One(
        "gnuhealth.imaging_worklist.worklist_template", "Worklist template"
    )


class TestResult(metaclass=PoolMeta):
    __name__ = "gnuhealth.imaging.test.result"

    merge_id = fields.Char("Merge ID")

    @classmethod
    def create(cls, vlist):
        Request = Pool().get('gnuhealth.imaging.test.request')
        vlist = [x.copy() for x in vlist]

        for values in vlist:
            request = Request.search(
                [("id", "=", values['request'])], limit=1)[0]

            if request:
                values['merge_id'] = request.merge_id or ''

        return super(TestResult, cls).create(vlist)



# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                   HEALTH INPATIENT CALENDAR package                   #
#               __init__.py test initialization                         #
#########################################################################
from .test_health_inpatient_calendar import suite


__all__ = ['suite']



#!/usr/bin/env python

# SPDX-FileCopyrightText: 2008-2025 Luis Falcón <falcon@gnuhealth.org>
# SPDX-FileCopyrightText: 2011-2025 GNU Solidario <health@gnusolidario.org>

# SPDX-License-Identifier: GPL-3.0-or-later

#########################################################################
#   Hospital Management Information System (HMIS) component of the      #
#                       GNU Health project                              #
#                   https://www.gnuhealth.org                           #
#########################################################################
#                    HEALTH NTD CHAGAS package                          #
#                health_ntd_chagas.py: main module                      #
#########################################################################

from datetime import datetime
from trytond.model import ModelView, ModelSQL, fields
from trytond.pyson import Eval, Not, Bool
from trytond.pool import Pool


__all__ = ['ChagasDUSurvey']


class ChagasDUSurvey(ModelSQL, ModelView):
    'Chagas DU Entomological Survey'
    __name__ = 'gnuhealth.chagas_du_survey'

    name = fields.Char('Survey Code', readonly=True)
    du = fields.Many2One('gnuhealth.du', 'DU', help="Domiciliary Unit")
    survey_date = fields.Date('Date', required=True)

    du_status = fields.Selection([
        (None, ''),
        ('initial', 'Initial'),
        ('unchanged', 'Unchanged'),
        ('better', 'Improved'),
        ('worse', 'Worsen'),
    ], 'Status',
        help="DU status compared to last visit", required=True, sort=False)

    # Findings of Triatomines in the DU
    triatomines = fields.Boolean(
        'Triatomines', help="Check this box if triatomines were found")
    vector = fields.Selection([
        (None, ''),
        ('t_infestans', 'T. infestans'),
        ('t_brasilensis', 'T. brasilensis'),
        ('r_prolixus', 'R. prolixus'),
        ('t_dimidiata', 'T. dimidiata'),
        ('p_megistus', 'P. megistus'),
    ], 'Vector', help="Vector", sort=False)

    nymphs = fields.Boolean(
        'Nymphs', "Check this box if triatomine nymphs were found")
    t_in_house = fields.Boolean(
        'Domiciliary',
        help="Check this box if triatomines were found inside the house")
    t_peri = fields.Boolean(
        'Peri-Domiciliary',
        help="Check this box if triatomines were found "
             "in the peridomiciliary area")

    # Infrastructure conditions
    dfloor = fields.Boolean(
        'Floor',
        help="Current floor can host triatomines")
    dwall = fields.Boolean(
        'Walls',
        help="Wall materials or state can host triatomines")
    droof = fields.Boolean(
        'Roof',
        help="Roof materials or state can host triatomines")
    dperi = fields.Boolean(
        'Peri-domicilary',
        help="Peri domiciliary area can host triatomines")

    # Preventive measures

    bugtraps = fields.Boolean(
        'Bug traps',
        help="The DU has traps to detect triatomines")

    # Chemical controls

    du_fumigation = fields.Boolean(
        'Fumigation',
        help="The DU has been fumigated")
    fumigation_date = fields.Date(
        'Fumigation Date',
        help="Last Fumigation Date",
        states={'invisible': Not(Bool(Eval('du_fumigation')))})

    du_paint = fields.Boolean(
        'Insecticide Paint',
        help="The DU has been treated with insecticide-containing paint")
    paint_date = fields.Date(
        'Paint Date',
        help="Last Paint Date",
        states={'invisible': Not(Bool(Eval('du_paint')))})

    observations = fields.Text('Observations')
    next_survey_date = fields.Date('Next survey')

    @staticmethod
    def default_survey_date():
        return datetime.now()

    @classmethod
    def generate_code(cls, **pattern):
        Config = Pool().get('gnuhealth.sequences')
        config = Config(1)
        sequence = config.get_multivalue(
            'chagas_du_survey_sequence', **pattern)
        if sequence:
            return sequence.get()

    @classmethod
    def create(cls, vlist):
        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('name'):
                values['name'] = cls.generate_code()
        return super(ChagasDUSurvey, cls).create(vlist)

    @classmethod
    def __setup__(cls):
        super(ChagasDUSurvey, cls).__setup__()

        # Do not cache default_key as it depends on time
        cls.__rpc__['default_get'].cache = None
